# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
import logging

from openlp.core.lib.settings import Settings

from PyQt4 import QtCore

log = logging.getLogger(__name__)

class MediaState(object):
    """
    An enumeration for possible States of the Media Player
    """
    Off = 0
    Loaded = 1
    Playing = 2
    Paused = 3
    Stopped = 4


class MediaType(object):
    """
    An enumeration of possibible Media Types
    """
    Unused = 0
    Audio = 1
    Video = 2
    CD = 3
    DVD = 4
    Folder = 5


class MediaInfo(object):
    """
    This class hold the media related infos
    """
    file_info = None
    volume = 100
    is_flash = False
    is_background = False
    length = 0
    start_time = 0
    end_time = 0
    media_type = MediaType()

def get_media_players():
    """
    This method extract the configured media players and overridden player from
    the settings.

    ``players_list``
       A list with all active media players.

    ``overridden_player``
        Here an special media player is chosen for all media actions.
    """
    log.debug(u'get_media_players')
    players = unicode(Settings().value(u'media/players').toString())
    if not players:
        players = u'webkit'
    reg_ex = QtCore.QRegExp(".*\[(.*)\].*")
    if Settings().value(u'media/override player',
        QtCore.QVariant(QtCore.Qt.Unchecked)).toInt()[0] == QtCore.Qt.Checked:
        if reg_ex.exactMatch(players):
            overridden_player = u'%s' % reg_ex.cap(1)
        else:
            overridden_player = u'auto'
    else:
        overridden_player = u''
    players_list = players.replace(u'[', u'').replace(u']', u'').split(u',')
    return players_list, overridden_player


def set_media_players(players_list, overridden_player=u'auto'):
    """
    This method saves the configured media players and overridden player to the
    settings

    ``players_list``
        A list with all active media players.

    ``overridden_player``
        Here an special media player is chosen for all media actions.
    """
    log.debug(u'set_media_players')
    players = u','.join(players_list)
    if Settings().value(u'media/override player',
        QtCore.QVariant(QtCore.Qt.Unchecked)).toInt()[0] == \
        QtCore.Qt.Checked and overridden_player != u'auto':
        players = players.replace(overridden_player, u'[%s]' % overridden_player)
    Settings().setValue(u'media/players', QtCore.QVariant(players))

from mediacontroller import MediaController



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging

from PyQt4 import QtCore, QtGui

from editcustomslidedialog import Ui_CustomSlideEditDialog

log = logging.getLogger(__name__)

class EditCustomSlideForm(QtGui.QDialog, Ui_CustomSlideEditDialog):
    """
    Class documentation goes here.
    """
    log.info(u'Custom Verse Editor loaded')
    def __init__(self, parent=None):
        """
        Constructor
        """
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        # Connecting signals and slots
        QtCore.QObject.connect(self.insertButton,
            QtCore.SIGNAL(u'clicked()'), self.onInsertButtonClicked)
        QtCore.QObject.connect(self.splitButton,
            QtCore.SIGNAL(u'clicked()'), self.onSplitButtonClicked)

    def setText(self, text):
        """
        Set the text for slideTextEdit.

        ``text``
            The text (unicode).
        """
        self.slideTextEdit.clear()
        if text:
            self.slideTextEdit.setPlainText(text)
        self.slideTextEdit.setFocus()

    def getText(self):
        """
        Returns a list with all slides.
        """
        return self.slideTextEdit.toPlainText().split(u'\n[===]\n')

    def onInsertButtonClicked(self):
        """
        Adds a slide split at the cursor.
        """
        self.insertSingleLineTextAtCursor(u'[===]')
        self.slideTextEdit.setFocus()

    def onSplitButtonClicked(self):
        """
        Adds an optional split at cursor.
        """
        self.insertSingleLineTextAtCursor(u'[---]')
        self.slideTextEdit.setFocus()

    def insertSingleLineTextAtCursor(self, text):
        """
        Adds ``text`` in a single line at the cursor position.
        """
        full_text = self.slideTextEdit.toPlainText()
        position = self.slideTextEdit.textCursor().position()
        if position and full_text[position-1] != u'\n':
             text = u'\n' + text
        if position ==  len(full_text) or full_text[position] != u'\n':
             text += u'\n'
        self.slideTextEdit.insertPlainText(text)




# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from datetime import datetime
from distutils.version import LooseVersion
import logging
import os
import sys

from PyQt4 import QtCore, QtGui

from openlp.core.lib import Receiver
from openlp.core.lib.settings import Settings
from openlp.core.lib.mediaplayer import MediaPlayer
from openlp.core.ui.media import MediaState

log = logging.getLogger(__name__)

VLC_AVAILABLE = False
try:
    import vlc
    VLC_AVAILABLE = bool(vlc.get_default_instance())
except (ImportError, NameError, NotImplementedError):
    pass
except OSError, e:
    if sys.platform.startswith('win'):
        if not isinstance(e, WindowsError) and e.winerror != 126:
            raise
    else:
        raise

if VLC_AVAILABLE:
    try:
        version = vlc.libvlc_get_version()
    except:
        version = u'0.0.0'
    if LooseVersion(version) < LooseVersion('1.1.0'):
        VLC_AVAILABLE = False
        log.debug(u'VLC could not be loaded: %s' % version)

AUDIO_EXT = [u'*.mp3', u'*.wav', u'*.wma', u'*.ogg']

VIDEO_EXT = [
    u'*.3gp',
    u'*.asf', u'*.wmv',
    u'*.au',
    u'*.avi',
    u'*.flv',
    u'*.mov',
    u'*.mp4', u'*.m4v',
    u'*.ogm', u'*.ogv',
    u'*.mkv', u'*.mka',
    u'*.ts', u'*.mpg',
    u'*.mpg', u'*.mp2',
    u'*.nsc',
    u'*.nsv',
    u'*.nut',
    u'*.ra', u'*.ram', u'*.rm', u'*.rv' ,u'*.rmbv',
    u'*.a52', u'*.dts', u'*.aac', u'*.flac' ,u'*.dv', u'*.vid',
    u'*.tta', u'*.tac',
    u'*.ty',
    u'*.dts',
    u'*.xa',
    u'*.iso',
    u'*.vob',
    u'*.webm',
    u'*.divx'
]


class VlcPlayer(MediaPlayer):
    """
    A specialised version of the MediaPlayer class, which provides a VLC
    display.
    """

    def __init__(self, parent):
        MediaPlayer.__init__(self, parent, u'vlc')
        self.original_name = u'VLC'
        self.display_name = u'&VLC'
        self.parent = parent
        self.canFolder = True
        self.audio_extensions_list = AUDIO_EXT
        self.video_extensions_list = VIDEO_EXT

    def setup(self, display):
        display.vlcWidget = QtGui.QFrame(display)
        # creating a basic vlc instance
        command_line_options = u'--no-video-title-show'
        if not display.hasAudio:
            command_line_options += u' --no-audio --no-video-title-show'
        if Settings().value(u'advanced/hide mouse',
            QtCore.QVariant(False)).toBool() and \
            display.controller.isLive:
            command_line_options += u' --mouse-hide-timeout=0'
        display.vlcInstance = vlc.Instance(command_line_options)
        display.vlcInstance.set_log_verbosity(2)
        # creating an empty vlc media player
        display.vlcMediaPlayer = display.vlcInstance.media_player_new()
        display.vlcWidget.resize(display.size())
        display.vlcWidget.raise_()
        display.vlcWidget.hide()
        # The media player has to be 'connected' to the QFrame.
        # (otherwise a video would be displayed in it's own window)
        # This is platform specific!
        # You have to give the id of the QFrame (or similar object)
        # to vlc, different platforms have different functions for this.
        win_id = int(display.vlcWidget.winId())
        if sys.platform == "win32":
            display.vlcMediaPlayer.set_hwnd(win_id)
        elif sys.platform == "darwin":
            # We have to use 'set_nsobject' since Qt4 on OSX uses Cocoa
            # framework and not the old Carbon.
            display.vlcMediaPlayer.set_nsobject(win_id)
        else:
            # for Linux using the X Server
            display.vlcMediaPlayer.set_xwindow(win_id)
        self.hasOwnWidget = True

    def check_available(self):
        return VLC_AVAILABLE

    def load(self, display):
        log.debug(u'load vid in Vlc Controller')
        controller = display.controller
        volume = controller.media_info.volume
        file_path = str(
            controller.media_info.file_info.absoluteFilePath().toUtf8())
        path = os.path.normcase(file_path)
        # create the media
        display.vlcMedia = display.vlcInstance.media_new_path(path)
        # put the media in the media player
        display.vlcMediaPlayer.set_media(display.vlcMedia)
        # parse the metadata of the file
        display.vlcMedia.parse()
        self.volume(display, volume)
        # We need to set media_info.length during load because we want
        # to avoid start and stop the video twice. Once for real playback
        # and once to just get media length.
        #
        # Media plugin depends on knowing media length before playback.
        controller.media_info.length = \
            int(display.vlcMediaPlayer.get_media().get_duration() / 1000)
        return True

    def media_state_wait(self, display, mediaState):
        """
        Wait for the video to change its state
        Wait no longer than 60 seconds. (loading an iso file needs a long time)
        """
        start = datetime.now()
        while not mediaState == display.vlcMedia.get_state():
            if display.vlcMedia.get_state() == vlc.State.Error:
                return False
            Receiver.send_message(u'openlp_process_events')
            if (datetime.now() - start).seconds > 60:
                return False
        return True

    def resize(self, display):
        display.vlcWidget.resize(display.size())

    def play(self, display):
        controller = display.controller
        start_time = 0
        if controller.media_info.start_time > 0:
            start_time = controller.media_info.start_time
        display.vlcMediaPlayer.play()
        if not self.media_state_wait(display, vlc.State.Playing):
            return False
        if start_time > 0:
            self.seek(display, controller.media_info.start_time * 1000)
        controller.media_info.length = \
            int(display.vlcMediaPlayer.get_media().get_duration() / 1000)
        controller.seekSlider.setMaximum(controller.media_info.length * 1000)
        self.state = MediaState.Playing
        display.vlcWidget.raise_()
        return True

    def pause(self, display):
        if display.vlcMedia.get_state() != vlc.State.Playing:
            return
        display.vlcMediaPlayer.pause()
        if self.media_state_wait(display, vlc.State.Paused):
            self.state = MediaState.Paused

    def stop(self, display):
        display.vlcMediaPlayer.stop()
        self.state = MediaState.Stopped

    def volume(self, display, vol):
        if display.hasAudio:
            display.vlcMediaPlayer.audio_set_volume(vol)

    def seek(self, display, seekVal):
        if display.vlcMediaPlayer.is_seekable():
            display.vlcMediaPlayer.set_time(seekVal)

    def reset(self, display):
        display.vlcMediaPlayer.stop()
        display.vlcWidget.setVisible(False)
        self.state = MediaState.Off

    def set_visible(self, display, status):
        if self.hasOwnWidget:
            display.vlcWidget.setVisible(status)

    def update_ui(self, display):
        # Stop video if playback is finished.
        if display.vlcMedia.get_state() == vlc.State.Ended:
            self.stop(display)
        controller = display.controller
        if controller.media_info.end_time > 0:
            if display.vlcMediaPlayer.get_time() > \
                controller.media_info.end_time * 1000:
                self.stop(display)
                self.set_visible(display, False)
        if not controller.seekSlider.isSliderDown():
            controller.seekSlider.setSliderPosition( \
                display.vlcMediaPlayer.get_time())




# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging

from PyQt4 import QtCore, QtGui

from openlp.core.lib import Receiver, SettingsTab, translate
from openlp.core.lib.ui import UiStrings, find_and_set_in_combo_box
from openlp.core.lib.settings import Settings
from openlp.plugins.bibles.lib import LayoutStyle, DisplayStyle, \
    update_reference_separators, get_reference_separator, LanguageSelection

log = logging.getLogger(__name__)


class BiblesTab(SettingsTab):
    """
    BiblesTab is the Bibles settings tab in the settings dialog.
    """
    log.info(u'Bible Tab loaded')

    def __init__(self, parent, title, visible_title, icon_path):
        self.paragraph_style = True
        self.show_new_chapters = False
        self.display_style = 0
        SettingsTab.__init__(self, parent, title, visible_title, icon_path)

    def setupUi(self):
        self.setObjectName(u'BiblesTab')
        SettingsTab.setupUi(self)
        self.verseDisplayGroupBox = QtGui.QGroupBox(self.leftColumn)
        self.verseDisplayGroupBox.setObjectName(u'verseDisplayGroupBox')
        self.verseDisplayLayout = QtGui.QFormLayout(self.verseDisplayGroupBox)
        self.verseDisplayLayout.setObjectName(u'verseDisplayLayout')
        self.newChaptersCheckBox = QtGui.QCheckBox(self.verseDisplayGroupBox)
        self.newChaptersCheckBox.setObjectName(u'newChaptersCheckBox')
        self.verseDisplayLayout.addRow(self.newChaptersCheckBox)
        self.displayStyleLabel = QtGui.QLabel(self.verseDisplayGroupBox)
        self.displayStyleLabel.setObjectName(u'displayStyleLabel')
        self.displayStyleComboBox = QtGui.QComboBox(self.verseDisplayGroupBox)
        self.displayStyleComboBox.addItems([u'', u'', u'', u''])
        self.displayStyleComboBox.setObjectName(u'displayStyleComboBox')
        self.verseDisplayLayout.addRow(self.displayStyleLabel,
            self.displayStyleComboBox)
        self.layoutStyleLabel = QtGui.QLabel(self.verseDisplayGroupBox)
        self.layoutStyleLabel.setObjectName(u'layoutStyleLabel')
        self.layoutStyleComboBox = QtGui.QComboBox(self.verseDisplayGroupBox)
        self.layoutStyleComboBox.setObjectName(u'layoutStyleComboBox')
        self.layoutStyleComboBox.addItems([u'', u'', u''])
        self.verseDisplayLayout.addRow(self.layoutStyleLabel,
            self.layoutStyleComboBox)
        self.bibleSecondCheckBox = QtGui.QCheckBox(self.verseDisplayGroupBox)
        self.bibleSecondCheckBox.setObjectName(u'bibleSecondCheckBox')
        self.verseDisplayLayout.addRow(self.bibleSecondCheckBox)
        self.bibleThemeLabel = QtGui.QLabel(self.verseDisplayGroupBox)
        self.bibleThemeLabel.setObjectName(u'BibleThemeLabel')
        self.bibleThemeComboBox = QtGui.QComboBox(self.verseDisplayGroupBox)
        self.bibleThemeComboBox.setSizeAdjustPolicy(
            QtGui.QComboBox.AdjustToMinimumContentsLength)
        self.bibleThemeComboBox.setSizePolicy(
            QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        self.bibleThemeComboBox.addItem(u'')
        self.bibleThemeComboBox.setObjectName(u'BibleThemeComboBox')
        self.verseDisplayLayout.addRow(self.bibleThemeLabel,
            self.bibleThemeComboBox)
        self.changeNoteLabel = QtGui.QLabel(self.verseDisplayGroupBox)
        self.changeNoteLabel.setWordWrap(True)
        self.changeNoteLabel.setObjectName(u'changeNoteLabel')
        self.verseDisplayLayout.addRow(self.changeNoteLabel)
        self.leftLayout.addWidget(self.verseDisplayGroupBox)
        self.scriptureReferenceGroupBox = QtGui.QGroupBox(self.leftColumn)
        self.scriptureReferenceGroupBox.setObjectName(
            u'scriptureReferenceGroupBox')
        self.scriptureReferenceLayout = QtGui.QGridLayout(
            self.scriptureReferenceGroupBox)
        self.verseSeparatorCheckBox = QtGui.QCheckBox(
            self.scriptureReferenceGroupBox)
        self.verseSeparatorCheckBox.setObjectName(u'verseSeparatorCheckBox')
        self.scriptureReferenceLayout.addWidget(self.verseSeparatorCheckBox, 0,
            0)
        self.verseSeparatorLineEdit = QtGui.QLineEdit(
            self.scriptureReferenceGroupBox)
#        self.verseSeparatorLineEdit.setPalette
        self.verseSeparatorLineEdit.setObjectName(u'verseSeparatorLineEdit')
        self.scriptureReferenceLayout.addWidget(self.verseSeparatorLineEdit, 0,
            1)
        self.rangeSeparatorCheckBox = QtGui.QCheckBox(
            self.scriptureReferenceGroupBox)
        self.rangeSeparatorCheckBox.setObjectName(u'rangeSeparatorCheckBox')
        self.scriptureReferenceLayout.addWidget(self.rangeSeparatorCheckBox, 1,
            0)
        self.rangeSeparatorLineEdit = QtGui.QLineEdit(
            self.scriptureReferenceGroupBox)
        self.rangeSeparatorLineEdit.setObjectName(u'rangeSeparatorLineEdit')
        self.scriptureReferenceLayout.addWidget(self.rangeSeparatorLineEdit, 1,
            1)
        self.listSeparatorCheckBox = QtGui.QCheckBox(
            self.scriptureReferenceGroupBox)
        self.listSeparatorCheckBox.setObjectName(u'listSeparatorCheckBox')
        self.scriptureReferenceLayout.addWidget(self.listSeparatorCheckBox, 2,
            0)
        self.listSeparatorLineEdit = QtGui.QLineEdit(
            self.scriptureReferenceGroupBox)
        self.listSeparatorLineEdit.setObjectName(u'listSeparatorLineEdit')
        self.scriptureReferenceLayout.addWidget(self.listSeparatorLineEdit, 2,
            1)
        self.endSeparatorCheckBox = QtGui.QCheckBox(
            self.scriptureReferenceGroupBox)
        self.endSeparatorCheckBox.setObjectName(u'endSeparatorCheckBox')
        self.scriptureReferenceLayout.addWidget(self.endSeparatorCheckBox, 3,
            0)
        self.endSeparatorLineEdit = QtGui.QLineEdit(
            self.scriptureReferenceGroupBox)
        self.endSeparatorLineEdit.setObjectName(u'endSeparatorLineEdit')
        self.endSeparatorLineEdit.setValidator(QtGui.QRegExpValidator(
            QtCore.QRegExp(r'[^0-9]*'), self.endSeparatorLineEdit))
        self.scriptureReferenceLayout.addWidget(self.endSeparatorLineEdit, 3,
            1)
        self.leftLayout.addWidget(self.scriptureReferenceGroupBox)
        self.rightColumn.setSizePolicy(
            QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        self.languageSelectionGroupBox = QtGui.QGroupBox(self.rightColumn)
        self.languageSelectionGroupBox.setObjectName(
            u'languageSelectionGroupBox')
        self.languageSelectionLayout = QtGui.QVBoxLayout(
            self.languageSelectionGroupBox)
        self.languageSelectionLabel = QtGui.QLabel(
            self.languageSelectionGroupBox)
        self.languageSelectionLabel.setObjectName(u'languageSelectionLabel')
        self.languageSelectionComboBox = QtGui.QComboBox(
            self.languageSelectionGroupBox)
        self.languageSelectionComboBox.setObjectName(
            u'languageSelectionComboBox')
        self.languageSelectionComboBox.addItems([u'', u'', u''])
        self.languageSelectionLayout.addWidget(self.languageSelectionLabel)
        self.languageSelectionLayout.addWidget(self.languageSelectionComboBox)
        self.rightLayout.addWidget(self.languageSelectionGroupBox)
        self.leftLayout.addStretch()
        self.rightLayout.addStretch()
        # Signals and slots
        QtCore.QObject.connect(
            self.newChaptersCheckBox, QtCore.SIGNAL(u'stateChanged(int)'),
            self.onNewChaptersCheckBoxChanged)
        QtCore.QObject.connect(
            self.displayStyleComboBox, QtCore.SIGNAL(u'activated(int)'),
            self.onDisplayStyleComboBoxChanged)
        QtCore.QObject.connect(
            self.bibleThemeComboBox, QtCore.SIGNAL(u'activated(int)'),
            self.onBibleThemeComboBoxChanged)
        QtCore.QObject.connect(
            self.layoutStyleComboBox, QtCore.SIGNAL(u'activated(int)'),
            self.onLayoutStyleComboBoxChanged)
        QtCore.QObject.connect(
            self.bibleSecondCheckBox, QtCore.SIGNAL(u'stateChanged(int)'),
            self.onBibleSecondCheckBox)
        QtCore.QObject.connect(
            self.verseSeparatorCheckBox, QtCore.SIGNAL(u'clicked(bool)'),
            self.onVerseSeparatorCheckBoxClicked)
        QtCore.QObject.connect(
            self.verseSeparatorLineEdit, QtCore.SIGNAL(u'textEdited(QString)'),
            self.onVerseSeparatorLineEditEdited)
        QtCore.QObject.connect(
            self.verseSeparatorLineEdit, QtCore.SIGNAL(u'editingFinished()'),
            self.onVerseSeparatorLineEditFinished)
        QtCore.QObject.connect(
            self.rangeSeparatorCheckBox, QtCore.SIGNAL(u'clicked(bool)'),
            self.onRangeSeparatorCheckBoxClicked)
        QtCore.QObject.connect(
            self.rangeSeparatorLineEdit, QtCore.SIGNAL(u'textEdited(QString)'),
            self.onRangeSeparatorLineEditEdited)
        QtCore.QObject.connect(
            self.rangeSeparatorLineEdit, QtCore.SIGNAL(u'editingFinished()'),
            self.onRangeSeparatorLineEditFinished)
        QtCore.QObject.connect(
            self.listSeparatorCheckBox, QtCore.SIGNAL(u'clicked(bool)'),
            self.onListSeparatorCheckBoxClicked)
        QtCore.QObject.connect(
            self.listSeparatorLineEdit, QtCore.SIGNAL(u'textEdited(QString)'),
            self.onListSeparatorLineEditEdited)
        QtCore.QObject.connect(
            self.listSeparatorLineEdit, QtCore.SIGNAL(u'editingFinished()'),
            self.onListSeparatorLineEditFinished)
        QtCore.QObject.connect(
            self.endSeparatorCheckBox, QtCore.SIGNAL(u'clicked(bool)'),
            self.onEndSeparatorCheckBoxClicked)
        QtCore.QObject.connect(
            self.endSeparatorLineEdit, QtCore.SIGNAL(u'textEdited(QString)'),
            self.onEndSeparatorLineEditEdited)
        QtCore.QObject.connect(
            self.endSeparatorLineEdit, QtCore.SIGNAL(u'editingFinished()'),
            self.onEndSeparatorLineEditFinished)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'theme_update_list'), self.updateThemeList)
        QtCore.QObject.connect(
            self.languageSelectionComboBox, QtCore.SIGNAL(u'activated(int)'),
            self.onLanguageSelectionComboBoxChanged)

    def retranslateUi(self):
        self.verseDisplayGroupBox.setTitle(
            translate('BiblesPlugin.BiblesTab', 'Verse Display'))
        self.newChaptersCheckBox.setText(
            translate('BiblesPlugin.BiblesTab',
            'Only show new chapter numbers'))
        self.layoutStyleLabel.setText(UiStrings().LayoutStyle)
        self.displayStyleLabel.setText(UiStrings().DisplayStyle)
        self.bibleThemeLabel.setText(
            translate('BiblesPlugin.BiblesTab', 'Bible theme:'))
        self.layoutStyleComboBox.setItemText(LayoutStyle.VersePerSlide,
            UiStrings().VersePerSlide)
        self.layoutStyleComboBox.setItemText(LayoutStyle.VersePerLine,
            UiStrings().VersePerLine)
        self.layoutStyleComboBox.setItemText(LayoutStyle.Continuous,
            UiStrings().Continuous)
        self.displayStyleComboBox.setItemText(DisplayStyle.NoBrackets,
            translate('BiblesPlugin.BiblesTab', 'No Brackets'))
        self.displayStyleComboBox.setItemText(DisplayStyle.Round,
            translate('BiblesPlugin.BiblesTab', '( And )'))
        self.displayStyleComboBox.setItemText(DisplayStyle.Curly,
            translate('BiblesPlugin.BiblesTab', '{ And }'))
        self.displayStyleComboBox.setItemText(DisplayStyle.Square,
            translate('BiblesPlugin.BiblesTab', '[ And ]'))
        self.changeNoteLabel.setText(translate('BiblesPlugin.BiblesTab',
            'Note:\nChanges do not affect verses already in the service.'))
        self.bibleSecondCheckBox.setText(
            translate('BiblesPlugin.BiblesTab', 'Display second Bible verses'))
        self.scriptureReferenceGroupBox.setTitle(
            translate('BiblesPlugin.BiblesTab', 'Custom Scripture References'))
        self.verseSeparatorCheckBox.setText(
            translate('BiblesPlugin.BiblesTab', 'Verse Separator:'))
        self.rangeSeparatorCheckBox.setText(
            translate('BiblesPlugin.BiblesTab', 'Range Separator:'))
        self.listSeparatorCheckBox.setText(
            translate('BiblesPlugin.BiblesTab', 'List Separator:'))
        self.endSeparatorCheckBox.setText(
            translate('BiblesPlugin.BiblesTab', 'End Mark:'))
        self.verseSeparatorLineEdit.setToolTip(
            translate('BiblesPlugin.BiblesTab', 'Multiple alternative '
                'verse separators may be defined.\nThey have to be separated '
                'by a vertical bar "|".\nPlease clear this edit line to use '
                'the default value.'))
        self.rangeSeparatorLineEdit.setToolTip(
            translate('BiblesPlugin.BiblesTab', 'Multiple alternative '
                'range separators may be defined.\nThey have to be separated '
                'by a vertical bar "|".\nPlease clear this edit line to use '
                'the default value.'))
        self.listSeparatorLineEdit.setToolTip(
            translate('BiblesPlugin.BiblesTab', 'Multiple alternative '
                'list separators may be defined.\nThey have to be separated '
                'by a vertical bar "|".\nPlease clear this edit line to use '
                'the default value.'))
        self.endSeparatorLineEdit.setToolTip(
            translate('BiblesPlugin.BiblesTab', 'Multiple alternative '
                'end marks may be defined.\nThey have to be separated by a '
                'vertical bar "|".\nPlease clear this edit line to use the '
                'default value.'))
        self.languageSelectionGroupBox.setTitle(translate(
            'BiblesPlugin.BiblesTab', 'Default Bible Language'))
        self.languageSelectionLabel.setText(translate('BiblesPlugin.BiblesTab',
            'Book name language in search field,\nsearch results and on '
            'display:'))
        self.languageSelectionComboBox.setItemText(LanguageSelection.Bible,
            translate('BiblesPlugin.BiblesTab', 'Bible Language'))
        self.languageSelectionComboBox.setItemText(
            LanguageSelection.Application,
            translate('BiblesPlugin.BiblesTab', 'Application Language'))
        self.languageSelectionComboBox.setItemText(LanguageSelection.English,
            translate('BiblesPlugin.BiblesTab', 'English'))

    def onBibleThemeComboBoxChanged(self):
        self.bible_theme = unicode(self.bibleThemeComboBox.currentText())

    def onDisplayStyleComboBoxChanged(self):
        self.display_style = self.displayStyleComboBox.currentIndex()

    def onLayoutStyleComboBoxChanged(self):
        self.layout_style = self.layoutStyleComboBox.currentIndex()

    def onLanguageSelectionComboBoxChanged(self):
        self.language_selection = self.languageSelectionComboBox.currentIndex()

    def onNewChaptersCheckBoxChanged(self, check_state):
        self.show_new_chapters = False
        # We have a set value convert to True/False.
        if check_state == QtCore.Qt.Checked:
            self.show_new_chapters = True

    def onBibleSecondCheckBox(self, check_state):
        self.second_bibles = False
        # We have a set value convert to True/False.
        if check_state == QtCore.Qt.Checked:
            self.second_bibles = True

    def onVerseSeparatorCheckBoxClicked(self, checked):
        if checked:
            self.verseSeparatorLineEdit.setFocus()
        else:
            self.verseSeparatorLineEdit.setText(
                get_reference_separator(u'sep_v_default'))
        self.verseSeparatorLineEdit.setPalette(
            self.getGreyTextPalette(not checked))

    def onVerseSeparatorLineEditEdited(self, text):
        self.verseSeparatorCheckBox.setChecked(True)
        self.verseSeparatorLineEdit.setPalette(
            self.getGreyTextPalette(False))

    def onVerseSeparatorLineEditFinished(self):
        if self.verseSeparatorLineEdit.isModified():
            text = self.verseSeparatorLineEdit.text()
            if text == get_reference_separator(u'sep_v_default') or \
                text.remove(u'|').isEmpty():
                self.verseSeparatorCheckBox.setChecked(False)
                self.verseSeparatorLineEdit.setText(
                    get_reference_separator(u'sep_v_default'))
                self.verseSeparatorLineEdit.setPalette(
                    self.getGreyTextPalette(True))

    def onRangeSeparatorCheckBoxClicked(self, checked):
        if checked:
            self.rangeSeparatorLineEdit.setFocus()
        else:
            self.rangeSeparatorLineEdit.setText(
                get_reference_separator(u'sep_r_default'))
        self.rangeSeparatorLineEdit.setPalette(
            self.getGreyTextPalette(not checked))

    def onRangeSeparatorLineEditEdited(self, text):
        self.rangeSeparatorCheckBox.setChecked(True)
        self.rangeSeparatorLineEdit.setPalette(
            self.getGreyTextPalette(False))

    def onRangeSeparatorLineEditFinished(self):
        if self.rangeSeparatorLineEdit.isModified():
            text = self.rangeSeparatorLineEdit.text()
            if text == get_reference_separator(u'sep_r_default') or \
                text.remove(u'|').isEmpty():
                self.rangeSeparatorCheckBox.setChecked(False)
                self.rangeSeparatorLineEdit.setText(
                    get_reference_separator(u'sep_r_default'))
                self.rangeSeparatorLineEdit.setPalette(
                    self.getGreyTextPalette(True))

    def onListSeparatorCheckBoxClicked(self, checked):
        if checked:
            self.listSeparatorLineEdit.setFocus()
        else:
            self.listSeparatorLineEdit.setText(
                get_reference_separator(u'sep_l_default'))
        self.listSeparatorLineEdit.setPalette(
            self.getGreyTextPalette(not checked))

    def onListSeparatorLineEditEdited(self, text):
        self.listSeparatorCheckBox.setChecked(True)
        self.listSeparatorLineEdit.setPalette(
            self.getGreyTextPalette(False))

    def onListSeparatorLineEditFinished(self):
        if self.listSeparatorLineEdit.isModified():
            text = self.listSeparatorLineEdit.text()
            if text == get_reference_separator(u'sep_l_default') or \
                text.remove(u'|').isEmpty():
                self.listSeparatorCheckBox.setChecked(False)
                self.listSeparatorLineEdit.setText(
                    get_reference_separator(u'sep_l_default'))
                self.listSeparatorLineEdit.setPalette(
                    self.getGreyTextPalette(True))

    def onEndSeparatorCheckBoxClicked(self, checked):
        if checked:
            self.endSeparatorLineEdit.setFocus()
        else:
            self.endSeparatorLineEdit.setText(
                get_reference_separator(u'sep_e_default'))
        self.endSeparatorLineEdit.setPalette(
            self.getGreyTextPalette(not checked))

    def onEndSeparatorLineEditEdited(self, text):
        self.endSeparatorCheckBox.setChecked(True)
        self.endSeparatorLineEdit.setPalette(
            self.getGreyTextPalette(False))

    def onEndSeparatorLineEditFinished(self):
        if self.endSeparatorLineEdit.isModified():
            text = self.endSeparatorLineEdit.text()
            if text == get_reference_separator(u'sep_e_default') or \
                text.remove(u'|').isEmpty():
                self.endSeparatorCheckBox.setChecked(False)
                self.endSeparatorLineEdit.setText(
                    get_reference_separator(u'sep_e_default'))
                self.endSeparatorLineEdit.setPalette(
                    self.getGreyTextPalette(True))

    def load(self):
        settings = Settings()
        settings.beginGroup(self.settingsSection)
        self.show_new_chapters = settings.value(
            u'display new chapter', QtCore.QVariant(False)).toBool()
        self.display_style = settings.value(
            u'display brackets', QtCore.QVariant(0)).toInt()[0]
        self.layout_style = settings.value(
            u'verse layout style', QtCore.QVariant(0)).toInt()[0]
        self.bible_theme = unicode(
            settings.value(u'bible theme', QtCore.QVariant(u'')).toString())
        self.second_bibles = settings.value(
            u'second bibles', QtCore.QVariant(True)).toBool()
        self.newChaptersCheckBox.setChecked(self.show_new_chapters)
        self.displayStyleComboBox.setCurrentIndex(self.display_style)
        self.layoutStyleComboBox.setCurrentIndex(self.layout_style)
        self.bibleSecondCheckBox.setChecked(self.second_bibles)
        verse_separator = unicode(settings.value(u'verse separator').toString())
        if (verse_separator.strip(u'|') == u'') or \
            (verse_separator == get_reference_separator(u'sep_v_default')):
            self.verseSeparatorLineEdit.setText(
                get_reference_separator(u'sep_v_default'))
            self.verseSeparatorLineEdit.setPalette(
                self.getGreyTextPalette(True))
            self.verseSeparatorCheckBox.setChecked(False)
        else:
            self.verseSeparatorLineEdit.setText(verse_separator)
            self.verseSeparatorLineEdit.setPalette(
                self.getGreyTextPalette(False))
            self.verseSeparatorCheckBox.setChecked(True)
        range_separator = unicode(settings.value(u'range separator').toString())
        if (range_separator.strip(u'|') == u'') or \
            (range_separator == get_reference_separator(u'sep_r_default')):
            self.rangeSeparatorLineEdit.setText(
                get_reference_separator(u'sep_r_default'))
            self.rangeSeparatorLineEdit.setPalette(
                self.getGreyTextPalette(True))
            self.rangeSeparatorCheckBox.setChecked(False)
        else:
            self.rangeSeparatorLineEdit.setText(range_separator)
            self.rangeSeparatorLineEdit.setPalette(
                self.getGreyTextPalette(False))
            self.rangeSeparatorCheckBox.setChecked(True)
        list_separator = unicode(settings.value(u'list separator').toString())
        if (list_separator.strip(u'|') == u'') or \
            (list_separator == get_reference_separator(u'sep_l_default')):
            self.listSeparatorLineEdit.setText(
                get_reference_separator(u'sep_l_default'))
            self.listSeparatorLineEdit.setPalette(
                self.getGreyTextPalette(True))
            self.listSeparatorCheckBox.setChecked(False)
        else:
            self.listSeparatorLineEdit.setText(list_separator)
            self.listSeparatorLineEdit.setPalette(
                self.getGreyTextPalette(False))
            self.listSeparatorCheckBox.setChecked(True)
        end_separator = unicode(settings.value(u'end separator').toString())
        if (end_separator.strip(u'|') == u'') or \
            (end_separator == get_reference_separator(u'sep_e_default')):
            self.endSeparatorLineEdit.setText(
                get_reference_separator(u'sep_e_default'))
            self.endSeparatorLineEdit.setPalette(
                self.getGreyTextPalette(True))
            self.endSeparatorCheckBox.setChecked(False)
        else:
            self.endSeparatorLineEdit.setText(end_separator)
            self.endSeparatorLineEdit.setPalette(
                self.getGreyTextPalette(False))
            self.endSeparatorCheckBox.setChecked(True)
        self.language_selection = settings.value(
            u'book name language', QtCore.QVariant(0)).toInt()[0]
        self.languageSelectionComboBox.setCurrentIndex(self.language_selection)
        settings.endGroup()

    def save(self):
        settings = Settings()
        settings.beginGroup(self.settingsSection)
        settings.setValue(u'display new chapter',
            QtCore.QVariant(self.show_new_chapters))
        settings.setValue(u'display brackets',
            QtCore.QVariant(self.display_style))
        settings.setValue(u'verse layout style',
            QtCore.QVariant(self.layout_style))
        settings.setValue(u'book name language',
            QtCore.QVariant(self.language_selection))
        settings.setValue(u'second bibles', QtCore.QVariant(self.second_bibles))
        settings.setValue(u'bible theme', QtCore.QVariant(self.bible_theme))
        if self.verseSeparatorCheckBox.isChecked():
            settings.setValue(u'verse separator',
                self.verseSeparatorLineEdit.text())
        else:
            settings.remove(u'verse separator')
        if self.rangeSeparatorCheckBox.isChecked():
            settings.setValue(u'range separator',
                self.rangeSeparatorLineEdit.text())
        else:
            settings.remove(u'range separator')
        if self.listSeparatorCheckBox.isChecked():
            settings.setValue(u'list separator',
                self.listSeparatorLineEdit.text())
        else:
            settings.remove(u'list separator')
        if self.endSeparatorCheckBox.isChecked():
            settings.setValue(u'end separator',
                self.endSeparatorLineEdit.text())
        else:
            settings.remove(u'end separator')
        update_reference_separators()
        Receiver.send_message(u'bibles_load_list')
        settings.endGroup()

    def updateThemeList(self, theme_list):
        """
        Called from ThemeManager when the Themes have changed.

        ``theme_list``
            The list of available themes::

                [u'Bible Theme', u'Song Theme']
        """
        self.bibleThemeComboBox.clear()
        self.bibleThemeComboBox.addItem(u'')
        for theme in theme_list:
            self.bibleThemeComboBox.addItem(theme)
        find_and_set_in_combo_box(self.bibleThemeComboBox, self.bible_theme)

    def getGreyTextPalette(self, greyed):
        """
        Returns a QPalette with greyed out text as used for placeholderText.
        """
        palette = QtGui.QPalette()
        color = self.palette().color(QtGui.QPalette.Active, QtGui.QPalette.Text)
        if greyed:
            color.setAlpha(128)
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.Text, color)
        return palette




# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from openlp.core.lib import build_icon, translate
from openlp.core.lib.ui import UiStrings, create_button_box, create_button
from openlp.plugins.songs.lib.ui import SongStrings


class Ui_EditSongDialog(object):
    def setupUi(self, editSongDialog):
        editSongDialog.setObjectName(u'editSongDialog')
        editSongDialog.resize(650, 400)
        editSongDialog.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        editSongDialog.setModal(True)
        self.dialogLayout = QtGui.QVBoxLayout(editSongDialog)
        self.dialogLayout.setSpacing(8)
        self.dialogLayout.setContentsMargins(8, 8, 8, 8)
        self.dialogLayout.setObjectName(u'dialogLayout')
        self.songTabWidget = QtGui.QTabWidget(editSongDialog)
        self.songTabWidget.setObjectName(u'songTabWidget')
        # lyrics tab
        self.lyricsTab = QtGui.QWidget()
        self.lyricsTab.setObjectName(u'lyricsTab')
        self.lyricsTabLayout = QtGui.QGridLayout(self.lyricsTab)
        self.lyricsTabLayout.setObjectName(u'lyricsTabLayout')
        self.titleLabel = QtGui.QLabel(self.lyricsTab)
        self.titleLabel.setObjectName(u'titleLabel')
        self.lyricsTabLayout.addWidget(self.titleLabel, 0, 0)
        self.titleEdit = QtGui.QLineEdit(self.lyricsTab)
        self.titleEdit.setObjectName(u'titleEdit')
        self.titleLabel.setBuddy(self.titleEdit)
        self.lyricsTabLayout.addWidget(self.titleEdit, 0, 1, 1, 2)
        self.alternativeTitleLabel = QtGui.QLabel(self.lyricsTab)
        self.alternativeTitleLabel.setObjectName(u'alternativeTitleLabel')
        self.lyricsTabLayout.addWidget(self.alternativeTitleLabel, 1, 0)
        self.alternativeEdit = QtGui.QLineEdit(self.lyricsTab)
        self.alternativeEdit.setObjectName(u'alternativeEdit')
        self.alternativeTitleLabel.setBuddy(self.alternativeEdit)
        self.lyricsTabLayout.addWidget(self.alternativeEdit, 1, 1, 1, 2)
        self.lyricsLabel = QtGui.QLabel(self.lyricsTab)
        self.lyricsLabel.setFixedHeight(self.titleEdit.sizeHint().height())
        self.lyricsLabel.setObjectName(u'lyricsLabel')
        self.lyricsTabLayout.addWidget(self.lyricsLabel, 2, 0,
            QtCore.Qt.AlignTop)
        self.verseListWidget = SingleColumnTableWidget(self.lyricsTab)
        self.verseListWidget.setAlternatingRowColors(True)
        self.verseListWidget.setSelectionBehavior(
            QtGui.QAbstractItemView.SelectRows)
        self.verseListWidget.setSelectionMode(
            QtGui.QAbstractItemView.SingleSelection)
        self.verseListWidget.setEditTriggers(
            QtGui.QAbstractItemView.NoEditTriggers)
        self.verseListWidget.setObjectName(u'verseListWidget')
        self.lyricsLabel.setBuddy(self.verseListWidget)
        self.lyricsTabLayout.addWidget(self.verseListWidget, 2, 1)
        self.verseOrderLabel = QtGui.QLabel(self.lyricsTab)
        self.verseOrderLabel.setObjectName(u'verseOrderLabel')
        self.lyricsTabLayout.addWidget(self.verseOrderLabel, 3, 0)
        self.verseOrderEdit = QtGui.QLineEdit(self.lyricsTab)
        self.verseOrderEdit.setObjectName(u'verseOrderEdit')
        self.verseOrderLabel.setBuddy(self.verseOrderEdit)
        self.lyricsTabLayout.addWidget(self.verseOrderEdit, 3, 1, 1, 2)
        self.verseButtonsLayout = QtGui.QVBoxLayout()
        self.verseButtonsLayout.setObjectName(u'verseButtonsLayout')
        self.verseAddButton = QtGui.QPushButton(self.lyricsTab)
        self.verseAddButton.setObjectName(u'verseAddButton')
        self.verseButtonsLayout.addWidget(self.verseAddButton)
        self.verseEditButton = QtGui.QPushButton(self.lyricsTab)
        self.verseEditButton.setObjectName(u'verseEditButton')
        self.verseButtonsLayout.addWidget(self.verseEditButton)
        self.verseEditAllButton = QtGui.QPushButton(self.lyricsTab)
        self.verseEditAllButton.setObjectName(u'verseEditAllButton')
        self.verseButtonsLayout.addWidget(self.verseEditAllButton)
        self.verseDeleteButton = QtGui.QPushButton(self.lyricsTab)
        self.verseDeleteButton.setObjectName(u'verseDeleteButton')
        self.verseButtonsLayout.addWidget(self.verseDeleteButton)
        self.verseButtonsLayout.addStretch()
        self.lyricsTabLayout.addLayout(self.verseButtonsLayout, 2, 2)
        self.songTabWidget.addTab(self.lyricsTab, u'')
        # authors tab
        self.authorsTab = QtGui.QWidget()
        self.authorsTab.setObjectName(u'authorsTab')
        self.authorsTabLayout = QtGui.QHBoxLayout(self.authorsTab)
        self.authorsTabLayout.setObjectName(u'authorsTabLayout')
        self.authorsLeftLayout = QtGui.QVBoxLayout()
        self.authorsLeftLayout.setObjectName(u'authorsLeftLayout')
        self.authorsGroupBox = QtGui.QGroupBox(self.authorsTab)
        self.authorsGroupBox.setObjectName(u'authorsGroupBox')
        self.authorsLayout = QtGui.QVBoxLayout(self.authorsGroupBox)
        self.authorsLayout.setObjectName(u'authorsLayout')
        self.authorAddLayout = QtGui.QHBoxLayout()
        self.authorAddLayout.setObjectName(u'authorAddLayout')
        self.authorsComboBox = editSongDialogComboBox(
            self.authorsGroupBox, u'authorsComboBox')
        self.authorAddLayout.addWidget(self.authorsComboBox)
        self.authorAddButton = QtGui.QPushButton(self.authorsGroupBox)
        self.authorAddButton.setObjectName(u'authorAddButton')
        self.authorAddLayout.addWidget(self.authorAddButton)
        self.authorsLayout.addLayout(self.authorAddLayout)
        self.authorsListView = QtGui.QListWidget(self.authorsGroupBox)
        self.authorsListView.setAlternatingRowColors(True)
        self.authorsListView.setObjectName(u'authorsListView')
        self.authorsLayout.addWidget(self.authorsListView)
        self.authorRemoveLayout = QtGui.QHBoxLayout()
        self.authorRemoveLayout.setObjectName(u'authorRemoveLayout')
        self.authorRemoveLayout.addStretch()
        self.authorRemoveButton = QtGui.QPushButton(self.authorsGroupBox)
        self.authorRemoveButton.setObjectName(u'authorRemoveButton')
        self.authorRemoveLayout.addWidget(self.authorRemoveButton)
        self.authorsLayout.addLayout(self.authorRemoveLayout)
        self.authorsLeftLayout.addWidget(self.authorsGroupBox)
        self.maintenanceLayout = QtGui.QHBoxLayout()
        self.maintenanceLayout.setObjectName(u'maintenanceLayout')
        self.maintenanceButton = QtGui.QPushButton(self.authorsTab)
        self.maintenanceButton.setObjectName(u'maintenanceButton')
        self.maintenanceLayout.addWidget(self.maintenanceButton)
        self.maintenanceLayout.addStretch()
        self.authorsLeftLayout.addLayout(self.maintenanceLayout)
        self.authorsTabLayout.addLayout(self.authorsLeftLayout)
        self.authorsRightLayout = QtGui.QVBoxLayout()
        self.authorsRightLayout.setObjectName(u'authorsRightLayout')
        self.topicsGroupBox = QtGui.QGroupBox(self.authorsTab)
        self.topicsGroupBox.setObjectName(u'topicsGroupBox')
        self.topicsLayout = QtGui.QVBoxLayout(self.topicsGroupBox)
        self.topicsLayout.setObjectName(u'topicsLayout')
        self.topicAddLayout = QtGui.QHBoxLayout()
        self.topicAddLayout.setObjectName(u'topicAddLayout')
        self.topicsComboBox = editSongDialogComboBox(
            self.topicsGroupBox, u'topicsComboBox')
        self.topicAddLayout.addWidget(self.topicsComboBox)
        self.topicAddButton = QtGui.QPushButton(self.topicsGroupBox)
        self.topicAddButton.setObjectName(u'topicAddButton')
        self.topicAddLayout.addWidget(self.topicAddButton)
        self.topicsLayout.addLayout(self.topicAddLayout)
        self.topicsListView = QtGui.QListWidget(self.topicsGroupBox)
        self.topicsListView.setAlternatingRowColors(True)
        self.topicsListView.setObjectName(u'topicsListView')
        self.topicsLayout.addWidget(self.topicsListView)
        self.topicRemoveLayout = QtGui.QHBoxLayout()
        self.topicRemoveLayout.setObjectName(u'topicRemoveLayout')
        self.topicRemoveLayout.addStretch()
        self.topicRemoveButton = QtGui.QPushButton(self.topicsGroupBox)
        self.topicRemoveButton.setObjectName(u'topicRemoveButton')
        self.topicRemoveLayout.addWidget(self.topicRemoveButton)
        self.topicsLayout.addLayout(self.topicRemoveLayout)
        self.authorsRightLayout.addWidget(self.topicsGroupBox)
        self.songBookGroupBox = QtGui.QGroupBox(self.authorsTab)
        self.songBookGroupBox.setObjectName(u'songBookGroupBox')
        self.songBookLayout = QtGui.QFormLayout(self.songBookGroupBox)
        self.songBookLayout.setObjectName(u'songBookLayout')
        self.songBookNameLabel = QtGui.QLabel(self.songBookGroupBox)
        self.songBookNameLabel.setObjectName(u'songBookNameLabel')
        self.songBookComboBox = editSongDialogComboBox(
            self.songBookGroupBox, u'songBookComboBox')
        self.songBookNameLabel.setBuddy(self.songBookComboBox)
        self.songBookLayout.addRow(self.songBookNameLabel,
            self.songBookComboBox)
        self.songBookNumberLabel = QtGui.QLabel(self.songBookGroupBox)
        self.songBookNumberLabel.setObjectName(u'songBookNumberLabel')
        self.songBookNumberEdit = QtGui.QLineEdit(self.songBookGroupBox)
        self.songBookNumberEdit.setObjectName(u'songBookNumberEdit')
        self.songBookNumberLabel.setBuddy(self.songBookNumberEdit)
        self.songBookLayout.addRow(self.songBookNumberLabel,
            self.songBookNumberEdit)
        self.authorsRightLayout.addWidget(self.songBookGroupBox)
        self.authorsTabLayout.addLayout(self.authorsRightLayout)
        self.songTabWidget.addTab(self.authorsTab, u'')
        # theme tab
        self.themeTab = QtGui.QWidget()
        self.themeTab.setObjectName(u'themeTab')
        self.themeTabLayout = QtGui.QHBoxLayout(self.themeTab)
        self.themeTabLayout.setObjectName(u'themeTabLayout')
        self.themeLeftLayout = QtGui.QVBoxLayout()
        self.themeLeftLayout.setObjectName(u'themeLeftLayout')
        self.themeGroupBox = QtGui.QGroupBox(self.themeTab)
        self.themeGroupBox.setObjectName(u'themeGroupBox')
        self.themeLayout = QtGui.QHBoxLayout(self.themeGroupBox)
        self.themeLayout.setObjectName(u'themeLayout')
        self.themeComboBox = editSongDialogComboBox(
            self.themeGroupBox, u'themeComboBox')
        self.themeLayout.addWidget(self.themeComboBox)
        self.themeAddButton = QtGui.QPushButton(self.themeGroupBox)
        self.themeAddButton.setObjectName(u'themeAddButton')
        self.themeLayout.addWidget(self.themeAddButton)
        self.themeLeftLayout.addWidget(self.themeGroupBox)
        self.rightsGroupBox = QtGui.QGroupBox(self.themeTab)
        self.rightsGroupBox.setObjectName(u'rightsGroupBox')
        self.rightsLayout = QtGui.QVBoxLayout(self.rightsGroupBox)
        self.rightsLayout.setObjectName(u'rightsLayout')
        self.copyrightLayout = QtGui.QHBoxLayout()
        self.copyrightLayout.setObjectName(u'copyrightLayout')
        self.copyrightEdit = QtGui.QLineEdit(self.rightsGroupBox)
        self.copyrightEdit.setObjectName(u'copyrightEdit')
        self.copyrightLayout.addWidget(self.copyrightEdit)
        self.copyrightInsertButton = QtGui.QToolButton(self.rightsGroupBox)
        self.copyrightInsertButton.setObjectName(u'copyrightInsertButton')
        self.copyrightLayout.addWidget(self.copyrightInsertButton)
        self.rightsLayout.addLayout(self.copyrightLayout)
        self.CCLILayout = QtGui.QHBoxLayout()
        self.CCLILayout.setObjectName(u'CCLILayout')
        self.CCLILabel = QtGui.QLabel(self.rightsGroupBox)
        self.CCLILabel.setObjectName(u'CCLILabel')
        self.CCLILayout.addWidget(self.CCLILabel)
        self.CCLNumberEdit = QtGui.QLineEdit(self.rightsGroupBox)
        self.CCLNumberEdit.setValidator(QtGui.QIntValidator())
        self.CCLNumberEdit.setObjectName(u'CCLNumberEdit')
        self.CCLILayout.addWidget(self.CCLNumberEdit)
        self.rightsLayout.addLayout(self.CCLILayout)
        self.themeLeftLayout.addWidget(self.rightsGroupBox)
        self.themeLeftLayout.addStretch()
        self.themeTabLayout.addLayout(self.themeLeftLayout)
        self.commentsGroupBox = QtGui.QGroupBox(self.themeTab)
        self.commentsGroupBox.setObjectName(u'commentsGroupBox')
        self.commentsLayout = QtGui.QVBoxLayout(self.commentsGroupBox)
        self.commentsLayout.setObjectName(u'commentsLayout')
        self.commentsEdit = QtGui.QTextEdit(self.commentsGroupBox)
        self.commentsEdit.setObjectName(u'commentsEdit')
        self.commentsLayout.addWidget(self.commentsEdit)
        self.themeTabLayout.addWidget(self.commentsGroupBox)
        self.songTabWidget.addTab(self.themeTab, u'')
        # audio tab
        self.audioTab = QtGui.QWidget()
        self.audioTab.setObjectName(u'audioTab')
        self.audioLayout = QtGui.QHBoxLayout(self.audioTab)
        self.audioLayout.setObjectName(u'audioLayout')
        self.audioListWidget = QtGui.QListWidget(self.audioTab)
        self.audioListWidget.setObjectName(u'audioListWidget')
        self.audioLayout.addWidget(self.audioListWidget)
        self.audioButtonsLayout = QtGui.QVBoxLayout()
        self.audioButtonsLayout.setObjectName(u'audioButtonsLayout')
        self.audioAddFromFileButton = QtGui.QPushButton(self.audioTab)
        self.audioAddFromFileButton.setObjectName(u'audioAddFromFileButton')
        self.audioButtonsLayout.addWidget(self.audioAddFromFileButton)
        self.audioAddFromMediaButton = QtGui.QPushButton(self.audioTab)
        self.audioAddFromMediaButton.setObjectName(u'audioAddFromMediaButton')
        self.audioButtonsLayout.addWidget(self.audioAddFromMediaButton)
        self.audioRemoveButton = QtGui.QPushButton(self.audioTab)
        self.audioRemoveButton.setObjectName(u'audioRemoveButton')
        self.audioButtonsLayout.addWidget(self.audioRemoveButton)
        self.audioRemoveAllButton = QtGui.QPushButton(self.audioTab)
        self.audioRemoveAllButton.setObjectName(u'audioRemoveAllButton')
        self.audioButtonsLayout.addWidget(self.audioRemoveAllButton)
        self.audioButtonsLayout.addStretch(1)
        self.upButton = create_button(self, u'upButton', role=u'up',
            click=self.onUpButtonClicked)
        self.downButton = create_button(self, u'downButton', role=u'down',
            click=self.onDownButtonClicked)
        self.audioButtonsLayout.addWidget(self.upButton)
        self.audioButtonsLayout.addWidget(self.downButton)
        self.audioLayout.addLayout(self.audioButtonsLayout)
        self.songTabWidget.addTab(self.audioTab, u'')
        # Last few bits
        self.dialogLayout.addWidget(self.songTabWidget)
        self.bottomLayout = QtGui.QHBoxLayout()
        self.bottomLayout.setObjectName(u'bottomLayout')
        self.warningLabel = QtGui.QLabel(editSongDialog)
        self.warningLabel.setObjectName(u'warningLabel')
        self.bottomLayout.addWidget(self.warningLabel)
        self.buttonBox = create_button_box(editSongDialog, u'buttonBox',
            [u'cancel', u'save'])
        self.bottomLayout.addWidget(self.buttonBox)
        self.dialogLayout.addLayout(self.bottomLayout)
        self.retranslateUi(editSongDialog)

    def retranslateUi(self, editSongDialog):
        editSongDialog.setWindowTitle(
            translate('SongsPlugin.EditSongForm', 'Song Editor'))
        self.titleLabel.setText(
            translate('SongsPlugin.EditSongForm', '&Title:'))
        self.alternativeTitleLabel.setText(
            translate('SongsPlugin.EditSongForm', 'Alt&ernate title:'))
        self.lyricsLabel.setText(
            translate('SongsPlugin.EditSongForm', '&Lyrics:'))
        self.verseOrderLabel.setText(
            translate('SongsPlugin.EditSongForm', '&Verse order:'))
        self.verseAddButton.setText(UiStrings().Add)
        self.verseEditButton.setText(UiStrings().Edit)
        self.verseEditAllButton.setText(
            translate('SongsPlugin.EditSongForm', 'Ed&it All'))
        self.verseDeleteButton.setText(UiStrings().Delete)
        self.songTabWidget.setTabText(
            self.songTabWidget.indexOf(self.lyricsTab),
            translate('SongsPlugin.EditSongForm', 'Title && Lyrics'))
        self.authorsGroupBox.setTitle(SongStrings.Authors)
        self.authorAddButton.setText(
            translate('SongsPlugin.EditSongForm', '&Add to Song'))
        self.authorRemoveButton.setText(
            translate('SongsPlugin.EditSongForm', '&Remove'))
        self.maintenanceButton.setText(translate('SongsPlugin.EditSongForm',
            '&Manage Authors, Topics, Song Books'))
        self.topicsGroupBox.setTitle(SongStrings.Topic)
        self.topicAddButton.setText(
            translate('SongsPlugin.EditSongForm', 'A&dd to Song'))
        self.topicRemoveButton.setText(
            translate('SongsPlugin.EditSongForm', 'R&emove'))
        self.songBookGroupBox.setTitle(SongStrings.SongBook)
        self.songBookNameLabel.setText(translate('SongsPlugin.EditSongForm',
            'Book:'))
        self.songBookNumberLabel.setText(translate('SongsPlugin.EditSongForm',
            'Number:'))
        self.songTabWidget.setTabText(
            self.songTabWidget.indexOf(self.authorsTab),
            translate('SongsPlugin.EditSongForm',
                'Authors, Topics && Song Book'))
        self.themeGroupBox.setTitle(UiStrings().Theme)
        self.themeAddButton.setText(
            translate('SongsPlugin.EditSongForm', 'New &Theme'))
        self.rightsGroupBox.setTitle(
            translate('SongsPlugin.EditSongForm', 'Copyright Information'))
        self.copyrightInsertButton.setText(SongStrings.CopyrightSymbol)
        self.CCLILabel.setText(UiStrings().CCLINumberLabel)
        self.commentsGroupBox.setTitle(
            translate('SongsPlugin.EditSongForm', 'Comments'))
        self.songTabWidget.setTabText(
            self.songTabWidget.indexOf(self.themeTab),
            translate('SongsPlugin.EditSongForm',
                'Theme, Copyright Info && Comments'))
        self.songTabWidget.setTabText(
            self.songTabWidget.indexOf(self.audioTab),
            translate('SongsPlugin.EditSongForm', 'Linked Audio'))
        self.audioAddFromFileButton.setText(
            translate('SongsPlugin.EditSongForm', 'Add &File(s)'))
        self.audioAddFromMediaButton.setText(
            translate('SongsPlugin.EditSongForm', 'Add &Media'))
        self.audioRemoveButton.setText(
            translate('SongsPlugin.EditSongForm', '&Remove'))
        self.audioRemoveAllButton.setText(
            translate('SongsPlugin.EditSongForm', 'Remove &All'))
        self.notAlllVersesUsed = translate('SongsPlugin.EditSongForm', '<strong>Warning:</strong>'
                ' Not all of the verses are in use.')
        self.noVerseOrder = translate('SongsPlugin.EditSongForm', '<strong>Warning:</strong>'
                ' You have not entered a verse order.')

def editSongDialogComboBox(parent, name):
    """
    Utility method to generate a standard combo box for this dialog.
    """
    comboBox = QtGui.QComboBox(parent)
    comboBox.setSizeAdjustPolicy(
        QtGui.QComboBox.AdjustToMinimumContentsLength)
    comboBox.setSizePolicy(
        QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
    comboBox.setEditable(True)
    comboBox.setInsertPolicy(QtGui.QComboBox.NoInsert)
    comboBox.setObjectName(name)
    return comboBox

class SingleColumnTableWidget(QtGui.QTableWidget):
    """
    Class to for a single column table widget to use for the verse table widget.
    """
    def __init__(self, parent):
        """
        Constructor
        """
        QtGui.QTableWidget.__init__(self, parent)
        self.horizontalHeader().setVisible(False)
        self.setColumnCount(1)

    def resizeEvent(self, event):
        """
        Resize the first column together with the widget.
        """
        QtGui.QTableWidget.resizeEvent(self, event)
        if self.columnCount():
            self.setColumnWidth(0, event.size().width())
            self.resizeRowsToContents()



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging

from PyQt4 import QtCore

from openlp.core.lib import Plugin, StringContent, build_icon, translate
from openlp.core.lib.settings import Settings
from openlp.plugins.media.lib import MediaMediaItem, MediaTab

log = logging.getLogger(__name__)

class MediaPlugin(Plugin):
    log.info(u'%s MediaPlugin loaded', __name__)

    def __init__(self, plugin_helpers):
        Plugin.__init__(self, u'media', plugin_helpers,
            MediaMediaItem)
        self.weight = -6
        self.iconPath = u':/plugins/plugin_media.png'
        self.icon = build_icon(self.iconPath)
        # passed with drag and drop messages
        self.dnd_id = u'Media'
        self.audio_extensions_list = \
            self.mediaController.get_audio_extensions_list()
        for ext in self.audio_extensions_list:
            self.serviceManager.supportedSuffixes(ext[2:])
        self.video_extensions_list = \
            self.mediaController.get_video_extensions_list()
        for ext in self.video_extensions_list:
            self.serviceManager.supportedSuffixes(ext[2:])

    def createSettingsTab(self, parent):
        """
        Create the settings Tab
        """
        visible_name = self.getString(StringContent.VisibleName)
        self.settingsTab = MediaTab(parent, self.name, visible_name[u'title'],
            self.mediaController.mediaPlayers, self.iconPath)

    def about(self):
        about_text = translate('MediaPlugin', '<strong>Media Plugin</strong>'
            '<br />The media plugin provides playback of audio and video.')
        return about_text

    def setPluginTextStrings(self):
        """
        Called to define all translatable texts of the plugin
        """
        ## Name PluginList ##
        self.textStrings[StringContent.Name] = {
            u'singular': translate('MediaPlugin', 'Media', 'name singular'),
            u'plural': translate('MediaPlugin', 'Media', 'name plural')
        }
        ## Name for MediaDockManager, SettingsManager ##
        self.textStrings[StringContent.VisibleName] = {
            u'title': translate('MediaPlugin', 'Media', 'container title')
        }
        # Middle Header Bar
        tooltips = {
            u'load': translate('MediaPlugin', 'Load new media.'),
            u'import': u'',
            u'new': translate('MediaPlugin', 'Add new media.'),
            u'edit': translate('MediaPlugin', 'Edit the selected media.'),
            u'delete': translate('MediaPlugin', 'Delete the selected media.'),
            u'preview': translate('MediaPlugin', 'Preview the selected media.'),
            u'live': translate('MediaPlugin', 'Send the selected media live.'),
            u'service': translate('MediaPlugin',
                'Add the selected media to the service.')
        }
        self.setPluginUiTextStrings(tooltips)

    def finalise(self):
        """
        Time to tidy up on exit
        """
        log.info(u'Media Finalising')
        self.mediaController.finalise()
        Plugin.finalise(self)

    def getDisplayCss(self):
        """
        Add css style sheets to htmlbuilder
        """
        return self.mediaController.get_media_display_css()

    def getDisplayJavaScript(self):
        """
        Add javascript functions to htmlbuilder
        """
        return self.mediaController.get_media_display_javascript()

    def getDisplayHtml(self):
        """
        Add html code to htmlbuilder
        """
        return self.mediaController.get_media_display_html()

    def appStartup(self):
        """
        Do a couple of things when the app starts up. In this particular case
        we want to check if we have the old "Use Phonon" setting, and convert
        it to "enable Phonon" and "make it the first one in the list".
        """
        settings = Settings()
        settings.beginGroup(self.settingsSection)
        if settings.contains(u'use phonon'):
            log.info(u'Found old Phonon setting')
            players = self.mediaController.mediaPlayers.keys()
            has_phonon = u'phonon' in players
            if settings.value(u'use phonon').toBool() and has_phonon:
                log.debug(u'Converting old setting to new setting')
                new_players = []
                if players:
                    new_players = [player for player in players \
                        if player != u'phonon']
                new_players.insert(0, u'phonon')
                self.mediaController.mediaPlayers[u'phonon'].isActive = True
                settings.setValue(u'players', \
                    QtCore.QVariant(u','.join(new_players)))
                self.settingsTab.load()
            settings.remove(u'use phonon')
        settings.endGroup()



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging
from datetime import datetime

from PyQt4 import QtCore, QtGui

from openlp.core.lib import Plugin, StringContent, Receiver, build_icon, \
    translate
from openlp.core.lib.db import Manager
from openlp.core.lib.ui import create_action
from openlp.core.lib.settings import Settings
from openlp.core.utils.actions import ActionList
from openlp.plugins.songusage.forms import SongUsageDetailForm, \
    SongUsageDeleteForm
from openlp.plugins.songusage.lib import upgrade
from openlp.plugins.songusage.lib.db import init_schema, SongUsageItem

log = logging.getLogger(__name__)

class SongUsagePlugin(Plugin):
    log.info(u'SongUsage Plugin loaded')

    def __init__(self, plugin_helpers):
        Plugin.__init__(self, u'songusage', plugin_helpers)
        self.manager = Manager(u'songusage', init_schema, upgrade_mod=upgrade)
        self.weight = -4
        self.icon = build_icon(u':/plugins/plugin_songusage.png')
        self.activeIcon = build_icon(u':/songusage/song_usage_active.png')
        self.inactiveIcon = build_icon(u':/songusage/song_usage_inactive.png')
        self.songUsageActive = False

    def checkPreConditions(self):
        return self.manager.session is not None

    def addToolsMenuItem(self, tools_menu):
        """
        Give the SongUsage plugin the opportunity to add items to the
        **Tools** menu.

        ``tools_menu``
            The actual **Tools** menu item, so that your actions can
            use it as their parent.
        """
        log.info(u'add tools menu')
        self.toolsMenu = tools_menu
        self.songUsageMenu = QtGui.QMenu(tools_menu)
        self.songUsageMenu.setObjectName(u'songUsageMenu')
        self.songUsageMenu.setTitle(translate(
            'SongUsagePlugin', '&Song Usage Tracking'))
        # SongUsage Delete
        self.songUsageDelete = create_action(tools_menu, u'songUsageDelete',
            text=translate('SongUsagePlugin', '&Delete Tracking Data'),
            statustip=translate('SongUsagePlugin',
            'Delete song usage data up to a specified date.'),
            triggers=self.onSongUsageDelete)
        # SongUsage Report
        self.songUsageReport = create_action(tools_menu, u'songUsageReport',
            text=translate('SongUsagePlugin', '&Extract Tracking Data'),
            statustip=translate('SongUsagePlugin',
            'Generate a report on song usage.'),
            triggers=self.onSongUsageReport)
        # SongUsage activation
        self.songUsageStatus = create_action(tools_menu, u'songUsageStatus',
            text=translate('SongUsagePlugin', 'Toggle Tracking'),
            statustip=translate('SongUsagePlugin',
            'Toggle the tracking of song usage.'), checked=False,
            shortcuts=[QtCore.Qt.Key_F4],
            triggers=self.toggleSongUsageState)
        # Add Menus together
        self.toolsMenu.addAction(self.songUsageMenu.menuAction())
        self.songUsageMenu.addAction(self.songUsageStatus)
        self.songUsageMenu.addSeparator()
        self.songUsageMenu.addAction(self.songUsageReport)
        self.songUsageMenu.addAction(self.songUsageDelete)
        self.songUsageActiveButton = QtGui.QToolButton(
            self.formParent.statusBar)
        self.songUsageActiveButton.setCheckable(True)
        self.songUsageActiveButton.setAutoRaise(True)
        self.songUsageActiveButton.setStatusTip(translate('SongUsagePlugin',
            'Toggle the tracking of song usage.'))
        self.songUsageActiveButton.setObjectName(u'songUsageActiveButton')
        self.formParent.statusBar.insertPermanentWidget(1,
            self.songUsageActiveButton)
        self.songUsageActiveButton.hide()
        # Signals and slots
        QtCore.QObject.connect(self.songUsageStatus,
            QtCore.SIGNAL(u'visibilityChanged(bool)'),
            self.songUsageStatus.setChecked)
        QtCore.QObject.connect(self.songUsageActiveButton,
            QtCore.SIGNAL(u'toggled(bool)'),
            self.toggleSongUsageState)
        self.songUsageMenu.menuAction().setVisible(False)

    def initialise(self):
        log.info(u'SongUsage Initialising')
        Plugin.initialise(self)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'slidecontroller_live_started'),
            self.displaySongUsage)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'print_service_started'),
            self.printSongUsage)
        self.songUsageActive = Settings().value(
            self.settingsSection + u'/active',
            QtCore.QVariant(False)).toBool()
        # Set the button and checkbox state
        self.setButtonState()
        action_list = ActionList.get_instance()
        action_list.add_action(self.songUsageStatus,
            unicode(translate('SongUsagePlugin', 'Song Usage')))
        action_list.add_action(self.songUsageDelete,
            unicode(translate('SongUsagePlugin', 'Song Usage')))
        action_list.add_action(self.songUsageReport,
            unicode(translate('SongUsagePlugin', 'Song Usage')))
        self.songUsageDeleteForm = SongUsageDeleteForm(self.manager,
            self.formParent)
        self.songUsageDetailForm = SongUsageDetailForm(self, self.formParent)
        self.songUsageMenu.menuAction().setVisible(True)
        self.songUsageActiveButton.show()

    def finalise(self):
        """
        Tidy up on exit
        """
        log.info(u'Plugin Finalise')
        self.manager.finalise()
        Plugin.finalise(self)
        self.songUsageMenu.menuAction().setVisible(False)
        action_list = ActionList.get_instance()
        action_list.remove_action(self.songUsageStatus,
            unicode(translate('SongUsagePlugin', 'Song Usage')))
        action_list.remove_action(self.songUsageDelete,
            unicode(translate('SongUsagePlugin', 'Song Usage')))
        action_list.remove_action(self.songUsageReport,
            unicode(translate('SongUsagePlugin', 'Song Usage')))
        self.songUsageActiveButton.hide()
        # stop any events being processed
        self.songUsageActive = False

    def toggleSongUsageState(self):
        """
        Manage the state of the audit collection and amend
        the UI when necessary,
        """
        self.songUsageActive = not self.songUsageActive
        Settings().setValue(self.settingsSection + u'/active',
            QtCore.QVariant(self.songUsageActive))
        self.setButtonState()

    def setButtonState(self):
        """
        Keep buttons inline.  Turn of signals to stop dead loop but we need the
        button and check box set correctly.
        """
        self.songUsageActiveButton.blockSignals(True)
        self.songUsageStatus.blockSignals(True)
        if self.songUsageActive:
            self.songUsageActiveButton.setIcon(self.activeIcon)
            self.songUsageStatus.setChecked(True)
            self.songUsageActiveButton.setChecked(True)
            self.songUsageActiveButton.setToolTip(translate('SongUsagePlugin',
                'Song usage tracking is active.'))
        else:
            self.songUsageActiveButton.setIcon(self.inactiveIcon)
            self.songUsageStatus.setChecked(False)
            self.songUsageActiveButton.setChecked(False)
            self.songUsageActiveButton.setToolTip(translate('SongUsagePlugin',
                'Song usage tracking is inactive.'))
        self.songUsageActiveButton.blockSignals(False)
        self.songUsageStatus.blockSignals(False)


    def displaySongUsage(self, item):
        """
        Song Usage for which has been displayed
        """
        self._add_song_usage(unicode(translate('SongUsagePlugin',
            'display')), item)

    def printSongUsage(self, item):
        """
        Song Usage for which has been printed
        """
        self._add_song_usage(unicode(translate('SongUsagePlugin',
            'printed')), item)

    def _add_song_usage(self, source, item):
        audit = item[0].audit
        if self.songUsageActive and audit:
            song_usage_item = SongUsageItem()
            song_usage_item.usagedate = datetime.today()
            song_usage_item.usagetime = datetime.now().time()
            song_usage_item.title = audit[0]
            song_usage_item.copyright = audit[2]
            song_usage_item.ccl_number = audit[3]
            song_usage_item.authors = u' '.join(audit[1])
            song_usage_item.plugin_name = item[0].name
            song_usage_item.source = source
            self.manager.save_object(song_usage_item)

    def onSongUsageDelete(self):
        self.songUsageDeleteForm.exec_()

    def onSongUsageReport(self):
        self.songUsageDetailForm.initialise()
        self.songUsageDetailForm.exec_()

    def about(self):
        about_text = translate('SongUsagePlugin', '<strong>SongUsage Plugin'
            '</strong><br />This plugin tracks the usage of songs in '
            'services.')
        return about_text

    def setPluginTextStrings(self):
        """
        Called to define all translatable texts of the plugin
        """
        ## Name PluginList ##
        self.textStrings[StringContent.Name] = {
            u'singular': translate('SongUsagePlugin', 'SongUsage',
                'name singular'),
            u'plural': translate('SongUsagePlugin', 'SongUsage',
                'name plural')
        }
        ## Name for MediaDockManager, SettingsManager ##
        self.textStrings[StringContent.VisibleName] = {
            u'title': translate('SongUsagePlugin', 'SongUsage',
                'container title')
        }



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

"""
Module implementing LanguageForm.
"""
import logging

from PyQt4.QtGui import QDialog

from openlp.core.lib import translate
from openlp.core.lib.ui import critical_error_message_box
from openlp.plugins.bibles.forms.languagedialog import \
    Ui_LanguageDialog
from openlp.plugins.bibles.lib.db import BiblesResourcesDB

log = logging.getLogger(__name__)

class LanguageForm(QDialog, Ui_LanguageDialog):
    """
    Class to manage a dialog which ask the user for a language.
    """
    log.info(u'LanguageForm loaded')

    def __init__(self, parent=None):
        """
        Constructor
        """
        QDialog.__init__(self, parent)
        self.setupUi(self)

    def exec_(self, bible_name):
        self.languageComboBox.addItem(u'')
        if bible_name:
            self.bibleLabel.setText(unicode(bible_name))
        items = BiblesResourcesDB.get_languages()
        self.languageComboBox.addItems([item[u'name'] for item in items])
        return QDialog.exec_(self)

    def accept(self):
        if not self.languageComboBox.currentText():
            critical_error_message_box(
                message=translate('BiblesPlugin.LanguageForm',
                'You need to choose a language.'))
            self.languageComboBox.setFocus()
            return False
        else:
            return QDialog.accept(self)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from openlp.core.lib import SettingsTab, translate, Receiver
from openlp.core.lib.ui import UiStrings, create_button
from openlp.core.lib.settings import Settings
from openlp.core.ui.media import get_media_players, set_media_players
class MediaQCheckBox(QtGui.QCheckBox):
    """
    MediaQCheckBox adds an extra property, playerName to the QCheckBox class.
    """
    def setPlayerName(self, name):
        self.playerName = name


class MediaTab(SettingsTab):
    """
    MediaTab is the Media settings tab in the settings dialog.
    """
    def __init__(self, parent, title, visible_title, media_players, icon_path):
        self.mediaPlayers = media_players
        self.savedUsedPlayers = None
        SettingsTab.__init__(self, parent, title, visible_title, icon_path)

    def setupUi(self):
        self.setObjectName(u'MediaTab')
        SettingsTab.setupUi(self)
        self.mediaPlayerGroupBox = QtGui.QGroupBox(self.leftColumn)
        self.mediaPlayerGroupBox.setObjectName(u'mediaPlayerGroupBox')
        self.mediaPlayerLayout = QtGui.QVBoxLayout(self.mediaPlayerGroupBox)
        self.mediaPlayerLayout.setObjectName(u'mediaPlayerLayout')
        self.playerCheckBoxes = {}
        for key, player in self.mediaPlayers.iteritems():
            player = self.mediaPlayers[key]
            checkbox = MediaQCheckBox(self.mediaPlayerGroupBox)
            checkbox.setEnabled(player.available)
            checkbox.setObjectName(player.name + u'CheckBox')
            self.playerCheckBoxes[player.name] = checkbox
            self.mediaPlayerLayout.addWidget(checkbox)
        self.leftLayout.addWidget(self.mediaPlayerGroupBox)
        self.playerOrderGroupBox = QtGui.QGroupBox(self.leftColumn)
        self.playerOrderGroupBox.setObjectName(u'playerOrderGroupBox')
        self.playerOrderLayout = QtGui.QHBoxLayout(self.playerOrderGroupBox)
        self.playerOrderLayout.setObjectName(u'playerOrderLayout')
        self.playerOrderlistWidget = QtGui.QListWidget( \
            self.playerOrderGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum,
            QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.playerOrderlistWidget. \
            sizePolicy().hasHeightForWidth())
        self.playerOrderlistWidget.setSizePolicy(sizePolicy)
        self.playerOrderlistWidget.setVerticalScrollBarPolicy( \
            QtCore.Qt.ScrollBarAsNeeded)
        self.playerOrderlistWidget.setHorizontalScrollBarPolicy( \
            QtCore.Qt.ScrollBarAlwaysOff)
        self.playerOrderlistWidget.setEditTriggers( \
            QtGui.QAbstractItemView.NoEditTriggers)
        self.playerOrderlistWidget.setObjectName(u'playerOrderlistWidget')
        self.playerOrderLayout.addWidget(self.playerOrderlistWidget)
        self.orderingButtonLayout = QtGui.QVBoxLayout()
        self.orderingButtonLayout.setObjectName(u'orderingButtonLayout')
        self.orderingButtonLayout.addStretch(1)
        self.orderingUpButton = create_button(self, u'orderingUpButton',
            role=u'up', click=self.onUpButtonClicked)
        self.orderingDownButton = create_button(self, u'orderingDownButton',
            role=u'down', click=self.onDownButtonClicked)
        self.orderingButtonLayout.addWidget(self.orderingUpButton)
        self.orderingButtonLayout.addWidget(self.orderingDownButton)
        self.orderingButtonLayout.addStretch(1)
        self.playerOrderLayout.addLayout(self.orderingButtonLayout)
        self.leftLayout.addWidget(self.playerOrderGroupBox)
        self.advancedGroupBox = QtGui.QGroupBox(self.leftColumn)
        self.advancedGroupBox.setObjectName(u'advancedGroupBox')
        self.advancedLayout = QtGui.QVBoxLayout(self.advancedGroupBox)
        self.advancedLayout.setObjectName(u'advancedLayout')
        self.overridePlayerCheckBox = QtGui.QCheckBox(self.advancedGroupBox)
        self.overridePlayerCheckBox.setObjectName(u'overridePlayerCheckBox')
        self.advancedLayout.addWidget(self.overridePlayerCheckBox)
        self.leftLayout.addWidget(self.advancedGroupBox)
        self.leftLayout.addStretch()
        self.rightLayout.addStretch()
        for key in self.mediaPlayers:
            player = self.mediaPlayers[key]
            checkbox = self.playerCheckBoxes[player.name]
            QtCore.QObject.connect(checkbox,
                QtCore.SIGNAL(u'stateChanged(int)'),
                self.onPlayerCheckBoxChanged)

    def retranslateUi(self):
        self.mediaPlayerGroupBox.setTitle(
            translate('MediaPlugin.MediaTab', 'Available Media Players'))
        for key in self.mediaPlayers:
            player = self.mediaPlayers[key]
            checkbox = self.playerCheckBoxes[player.name]
            checkbox.setPlayerName(player.name)
            if player.available:
                checkbox.setText(player.display_name)
            else:
                checkbox.setText(
                    unicode(translate('MediaPlugin.MediaTab',
                    '%s (unavailable)')) % player.display_name)
        self.playerOrderGroupBox.setTitle(
            translate('MediaPlugin.MediaTab', 'Player Order'))
        self.advancedGroupBox.setTitle(UiStrings().Advanced)
        self.overridePlayerCheckBox.setText(
            translate('MediaPlugin.MediaTab',
            'Allow media player to be overridden'))

    def onPlayerCheckBoxChanged(self, check_state):
        player = self.sender().playerName
        if check_state == QtCore.Qt.Checked:
            if player not in self.usedPlayers:
                self.usedPlayers.append(player)
        else:
            if player in self.usedPlayers:
                self.usedPlayers.remove(player)
        self.updatePlayerList()

    def updatePlayerList(self):
        self.playerOrderlistWidget.clear()
        for player in self.usedPlayers:
            if player in self.playerCheckBoxes.keys():
                if len(self.usedPlayers) == 1:
                    # At least one media player has to stay active
                    self.playerCheckBoxes[u'%s' % player].setEnabled(False)
                else:
                    self.playerCheckBoxes[u'%s' % player].setEnabled(True)
                self.playerOrderlistWidget.addItem(
                    self.mediaPlayers[unicode(player)].original_name)

    def onUpButtonClicked(self):
        row = self.playerOrderlistWidget.currentRow()
        if row <= 0:
            return
        item = self.playerOrderlistWidget.takeItem(row)
        self.playerOrderlistWidget.insertItem(row - 1, item)
        self.playerOrderlistWidget.setCurrentRow(row - 1)
        self.usedPlayers.insert(row - 1, self.usedPlayers.pop(row))

    def onDownButtonClicked(self):
        row = self.playerOrderlistWidget.currentRow()
        if row == -1 or row > self.playerOrderlistWidget.count() - 1:
            return
        item = self.playerOrderlistWidget.takeItem(row)
        self.playerOrderlistWidget.insertItem(row + 1, item)
        self.playerOrderlistWidget.setCurrentRow(row + 1)
        self.usedPlayers.insert(row + 1, self.usedPlayers.pop(row))

    def load(self):
        if self.savedUsedPlayers:
            self.usedPlayers = self.savedUsedPlayers
        self.usedPlayers = get_media_players()[0]
        self.savedUsedPlayers = self.usedPlayers
        for key in self.mediaPlayers:
            player = self.mediaPlayers[key]
            checkbox = self.playerCheckBoxes[player.name]
            if player.available and player.name in self.usedPlayers:
                checkbox.setChecked(True)
            else:
                checkbox.setChecked(False)
        self.updatePlayerList()
        self.overridePlayerCheckBox.setChecked(Settings().value(
            self.settingsSection + u'/override player',
            QtCore.QVariant(QtCore.Qt.Unchecked)).toInt()[0])

    def save(self):
        override_changed = False
        player_string_changed = False
        old_players, override_player = get_media_players()
        if self.usedPlayers != old_players:
            # clean old Media stuff
            set_media_players(self.usedPlayers, override_player)
            player_string_changed = True
            override_changed = True
        setting_key = self.settingsSection + u'/override player'
        if Settings().value(setting_key).toInt()[0] != \
            self.overridePlayerCheckBox.checkState():
            Settings().setValue(setting_key,
                QtCore.QVariant(self.overridePlayerCheckBox.checkState()))
            override_changed = True
        if override_changed:
            Receiver.send_message(u'mediaitem_media_rebuild')
        if player_string_changed:
            Receiver.send_message(u'mediaitem_media_rebuild')
            Receiver.send_message(u'config_screen_changed')



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The :mod:`db` module provides the database and schema that is the backend for
the Custom plugin
"""

from sqlalchemy import Column, Table, types
from sqlalchemy.orm import mapper

from openlp.core.lib.db import BaseModel, init_db
from openlp.core.utils import locale_compare

class CustomSlide(BaseModel):
    """
    CustomSlide model
    """
    # By default sort the customs by its title considering language specific
    # characters.
    def __lt__(self, other):
        r = locale_compare(self.title, other.title)
        return True if r < 0 else False

    def __eq__(self, other):
        return 0 == locale_compare(self.title, other.title)


def init_schema(url):
    """
    Setup the custom database connection and initialise the database schema

    ``url``
        The database to setup
    """
    session, metadata = init_db(url)

    custom_slide_table = Table(u'custom_slide', metadata,
        Column(u'id', types.Integer(), primary_key=True),
        Column(u'title', types.Unicode(255), nullable=False),
        Column(u'text', types.UnicodeText, nullable=False),
        Column(u'credits', types.UnicodeText),
        Column(u'theme_name', types.Unicode(128))
    )

    mapper(CustomSlide, custom_slide_table)

    metadata.create_all(checkfirst=True)
    return session



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The :mod:`olpimport` module provides the functionality for importing OpenLP
song databases into the current installation database.
"""
import logging

from sqlalchemy import create_engine, MetaData, Table
from sqlalchemy.orm import class_mapper, mapper, relation, scoped_session, \
    sessionmaker
from sqlalchemy.orm.exc import UnmappedClassError

from openlp.core.lib import translate
from openlp.core.lib.db import BaseModel
from openlp.core.ui.wizard import WizardStrings
from openlp.plugins.songs.lib import clean_song
from openlp.plugins.songs.lib.db import Author, Book, Song, Topic, MediaFile
from songimport import SongImport

log = logging.getLogger(__name__)

class OpenLPSongImport(SongImport):
    """
    The :class:`OpenLPSongImport` class provides OpenLP with the ability to
    import song databases from other installations of OpenLP.
    """
    def __init__(self, manager, **kwargs):
        """
        Initialise the import.

        ``manager``
            The song manager for the running OpenLP installation.

        ``source_db``
            The database providing the data to import.
        """
        SongImport.__init__(self, manager, **kwargs)
        self.sourceSession = None

    def doImport(self, progressDialog=None):
        """
        Run the import for an OpenLP version 2 song database.

        ``progressDialog``
            The QProgressDialog used when importing songs from the FRW.
        """

        class OldAuthor(BaseModel):
            """
            Author model
            """
            pass


        class OldBook(BaseModel):
            """
            Book model
            """
            pass


        class OldMediaFile(BaseModel):
            """
            MediaFile model
            """
            pass


        class OldSong(BaseModel):
            """
            Song model
            """
            pass


        class OldTopic(BaseModel):
            """
            Topic model
            """
            pass

        # Check the file type
        if not self.importSource.endswith(u'.sqlite'):
            self.logError(self.importSource,
                translate('SongsPlugin.OpenLPSongImport',
                'Not a valid OpenLP 2.0 song database.'))
            return
        self.importSource = u'sqlite:///%s' % self.importSource
        # Load the db file
        engine = create_engine(self.importSource)
        source_meta = MetaData()
        source_meta.reflect(engine)
        self.sourceSession = scoped_session(sessionmaker(bind=engine))
        if u'media_files' in source_meta.tables.keys():
            has_media_files = True
        else:
            has_media_files = False
        source_authors_table = source_meta.tables[u'authors']
        source_song_books_table = source_meta.tables[u'song_books']
        source_songs_table = source_meta.tables[u'songs']
        source_topics_table = source_meta.tables[u'topics']
        source_authors_songs_table = source_meta.tables[u'authors_songs']
        source_songs_topics_table = source_meta.tables[u'songs_topics']
        source_media_files_songs_table = None
        if has_media_files:
            source_media_files_table = source_meta.tables[u'media_files']
            source_media_files_songs_table = \
                source_meta.tables.get(u'media_files_songs')
            try:
                class_mapper(OldMediaFile)
            except UnmappedClassError:
                mapper(OldMediaFile, source_media_files_table)
        song_props = {
            'authors': relation(OldAuthor, backref='songs',
            secondary=source_authors_songs_table),
            'book': relation(OldBook, backref='songs'),
            'topics': relation(OldTopic, backref='songs',
            secondary=source_songs_topics_table)
        }
        if has_media_files:
            if isinstance(source_media_files_songs_table, Table):
                song_props['media_files'] = relation(OldMediaFile,
                    backref='songs',
                    secondary=source_media_files_songs_table)
            else:
                song_props['media_files'] = relation(OldMediaFile,
                    backref='songs',
                    foreign_keys=[source_media_files_table.c.song_id],
                    primaryjoin=source_songs_table.c.id == \
                        source_media_files_table.c.song_id)
        try:
            class_mapper(OldAuthor)
        except UnmappedClassError:
            mapper(OldAuthor, source_authors_table)
        try:
            class_mapper(OldBook)
        except UnmappedClassError:
            mapper(OldBook, source_song_books_table)
        try:
            class_mapper(OldSong)
        except UnmappedClassError:
            mapper(OldSong, source_songs_table, properties=song_props)
        try:
            class_mapper(OldTopic)
        except UnmappedClassError:
            mapper(OldTopic, source_topics_table)

        source_songs = self.sourceSession.query(OldSong).all()
        if self.importWizard:
            self.importWizard.progressBar.setMaximum(len(source_songs))
        for song in source_songs:
            new_song = Song()
            new_song.title = song.title
            if has_media_files and hasattr(song, 'alternate_title'):
                new_song.alternate_title = song.alternate_title
            else:
                old_titles = song.search_title.split(u'@')
                if len(old_titles) > 1:
                    new_song.alternate_title = old_titles[1]
            # Values will be set when cleaning the song.
            new_song.search_title = u''
            new_song.search_lyrics = u''
            new_song.song_number = song.song_number
            new_song.lyrics = song.lyrics
            new_song.verse_order = song.verse_order
            new_song.copyright = song.copyright
            new_song.comments = song.comments
            new_song.theme_name = song.theme_name
            new_song.ccli_number = song.ccli_number
            for author in song.authors:
                existing_author = self.manager.get_object_filtered(
                    Author, Author.display_name == author.display_name)
                if existing_author is None:
                    existing_author = Author.populate(
                        first_name=author.first_name,
                        last_name=author.last_name,
                        display_name=author.display_name)
                new_song.authors.append(existing_author)
            if song.book:
                existing_song_book = self.manager.get_object_filtered(
                    Book, Book.name == song.book.name)
                if existing_song_book is None:
                    existing_song_book = Book.populate(name=song.book.name,
                        publisher=song.book.publisher)
                new_song.book = existing_song_book
            if song.topics:
                for topic in song.topics:
                    existing_topic = self.manager.get_object_filtered(
                        Topic, Topic.name == topic.name)
                    if existing_topic is None:
                        existing_topic = Topic.populate(name=topic.name)
                    new_song.topics.append(existing_topic)
            if has_media_files:
                if song.media_files:
                    for media_file in song.media_files:
                        existing_media_file = \
                            self.manager.get_object_filtered(MediaFile,
                                MediaFile.file_name == media_file.file_name)
                        if existing_media_file:
                            new_song.media_files.append(existing_media_file)
                        else:
                            new_song.media_files.append(MediaFile.populate(
                                file_name=media_file.file_name))
            clean_song(self.manager, new_song)
            self.manager.save_object(new_song)
            if progressDialog:
                progressDialog.setValue(progressDialog.value() + 1)
                progressDialog.setLabelText(
                    WizardStrings.ImportingType % new_song.title)
            else:
                self.importWizard.incrementProgressBar(
                    WizardStrings.ImportingType % new_song.title)
            if self.stopImportFlag:
                break
        engine.dispose()



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The :mod:`upgrade` module provides a way for the database and schema that is the
backend for the Songs plugin
"""
import logging

from sqlalchemy import Column, Table, types
from sqlalchemy.exc import NoSuchTableError, OperationalError
from sqlalchemy.sql.expression import func
from migrate.changeset.constraint import ForeignKeyConstraint

log = logging.getLogger(__name__)
__version__ = 3


def upgrade_setup(metadata):
    """
    Set up the latest revision all tables, with reflection, needed for the
    upgrade process. If you want to drop a table, you need to remove it from
    here, and add it to your upgrade function.

    :param metadata: The SQLAlchemy metadata object
    """
    tables = {
        u'authors': Table(u'authors', metadata, autoload=True),
        u'media_files': Table(u'media_files', metadata, autoload=True),
        u'song_books': Table(u'song_books', metadata, autoload=True),
        u'songs': Table(u'songs', metadata, autoload=True),
        u'topics': Table(u'topics', metadata, autoload=True),
        u'authors_songs': Table(u'authors_songs', metadata, autoload=True),
        u'songs_topics': Table(u'songs_topics', metadata, autoload=True)
    }
    return tables


def upgrade_1(session, metadata, tables):
    """
    Version 1 upgrade.

    This upgrade removes the many-to-many relationship between songs and
    media_files and replaces it with a one-to-many, which is far more
    representative of the real relationship between the two entities.

    In order to facilitate this one-to-many relationship, a song_id column is
    added to the media_files table, and a weight column so that the media
    files can be ordered.

    :param session: An SQLAlchemy Session object
    :param metadata: An SQLAlchemy MetaData object
    :param tables: A dictionary of tables
    """
    try:
        Table(u'media_files_songs', metadata, autoload=True).drop(checkfirst=True)
        Column(u'song_id', types.Integer(), default=None)\
            .create(table=tables[u'media_files'])
        Column(u'weight', types.Integer(), default=0)\
            .create(table=tables[u'media_files'])
        if metadata.bind.url.get_dialect().name != 'sqlite':
            # SQLite doesn't support ALTER TABLE ADD CONSTRAINT
            ForeignKeyConstraint([u'song_id'], [u'songs.id'],
                table=tables[u'media_files']).create()
    except (NoSuchTableError, OperationalError):
        log.info(u'Upgrade 1 has already been run, continue with upgrade')


def upgrade_2(session, metadata, tables):
    """
    Version 2 upgrade.

    This upgrade adds a create_date and last_modified date to the songs table

    :param session: An SQLAlchemy Session object
    :param metadata: An SQLAlchemy MetaData object
    :param tables: A dictionary of tables
    """
    try:
        Column(u'create_date', types.DateTime(), default=func.now())\
            .create(table=tables[u'songs'])
        Column(u'last_modified', types.DateTime(), default=func.now())\
            .create(table=tables[u'songs'])
    except OperationalError:
        log.info(u'Upgrade 2 has already been run, continue with upgrade')


def upgrade_3(session, metadata, tables):
    """
    Version 3 upgrade.

    This upgrade adds a temporary song flag to the songs table

    :param session: An SQLAlchemy Session object
    :param metadata: An SQLAlchemy MetaData object
    :param tables: A dictionary of tables
    """
    try:
        Column(u'temporary', types.Boolean(), default=False)\
            .create(table=tables[u'songs'])
    except OperationalError:
        log.info(u'Upgrade 3 has already been run, continue with upgrade')



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from openlp.core.lib import build_icon
from openlp.core.lib.ui import UiStrings, create_button_box
from openlp.plugins.songs.lib.ui import SongStrings


class Ui_SongMaintenanceDialog(object):
    def setupUi(self, songMaintenanceDialog):
        songMaintenanceDialog.setObjectName(u'songMaintenanceDialog')
        songMaintenanceDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        songMaintenanceDialog.resize(10, 350)
        songMaintenanceDialog.setWindowIcon(
            build_icon(u':/icon/openlp-logo.svg'))
        self.dialogLayout = QtGui.QGridLayout(songMaintenanceDialog)
        self.dialogLayout.setObjectName(u'dialogLayout')
        self.typeListWidget = QtGui.QListWidget(songMaintenanceDialog)
        self.typeListWidget.setIconSize(QtCore.QSize(32, 32))
        self.typeListWidget.setUniformItemSizes(True)
        self.typeListWidget.setObjectName(u'typeListWidget')
        self.listItemAuthors = QtGui.QListWidgetItem(self.typeListWidget)
        self.listItemAuthors.setIcon(
            build_icon(u':/songs/author_maintenance.png'))
        self.listItemTopics = QtGui.QListWidgetItem(self.typeListWidget)
        self.listItemTopics.setIcon(
            build_icon(u':/songs/topic_maintenance.png'))
        self.listItemBooks = QtGui.QListWidgetItem(self.typeListWidget)
        self.listItemBooks.setIcon(
            build_icon(u':/songs/book_maintenance.png'))
        self.dialogLayout.addWidget(self.typeListWidget, 0, 0)
        self.stackedLayout = QtGui.QStackedLayout()
        self.stackedLayout.setObjectName(u'stackedLayout')
        # authors page
        self.authorsPage = QtGui.QWidget(songMaintenanceDialog)
        self.authorsPage.setObjectName(u'authorsPage')
        self.authorsLayout = QtGui.QVBoxLayout(self.authorsPage)
        self.authorsLayout.setObjectName(u'authorsLayout')
        self.authorsListWidget = QtGui.QListWidget(self.authorsPage)
        self.authorsListWidget.setObjectName(u'authorsListWidget')
        self.authorsLayout.addWidget(self.authorsListWidget)
        self.authorsButtonsLayout = QtGui.QHBoxLayout()
        self.authorsButtonsLayout.setObjectName(u'authorsButtonsLayout')
        self.authorsButtonsLayout.addStretch()
        self.authorsAddButton = QtGui.QPushButton(self.authorsPage)
        self.authorsAddButton.setIcon(build_icon(u':/songs/author_add.png'))
        self.authorsAddButton.setObjectName(u'authorsAddButton')
        self.authorsButtonsLayout.addWidget(self.authorsAddButton)
        self.authorsEditButton = QtGui.QPushButton(self.authorsPage)
        self.authorsEditButton.setIcon(build_icon(u':/songs/author_edit.png'))
        self.authorsEditButton.setObjectName(u'authorsEditButton')
        self.authorsButtonsLayout.addWidget(self.authorsEditButton)
        self.authorsDeleteButton = QtGui.QPushButton(self.authorsPage)
        self.authorsDeleteButton.setIcon(
            build_icon(u':/songs/author_delete.png'))
        self.authorsDeleteButton.setObjectName(u'authorsDeleteButton')
        self.authorsButtonsLayout.addWidget(self.authorsDeleteButton)
        self.authorsLayout.addLayout(self.authorsButtonsLayout)
        self.stackedLayout.addWidget(self.authorsPage)
        # topics page
        self.topicsPage = QtGui.QWidget(songMaintenanceDialog)
        self.topicsPage.setObjectName(u'topicsPage')
        self.topicsLayout = QtGui.QVBoxLayout(self.topicsPage)
        self.topicsLayout.setObjectName(u'topicsLayout')
        self.topicsListWidget = QtGui.QListWidget(self.topicsPage)
        self.topicsListWidget.setObjectName(u'topicsListWidget')
        self.topicsLayout.addWidget(self.topicsListWidget)
        self.topicsButtonsLayout = QtGui.QHBoxLayout()
        self.topicsButtonsLayout.setObjectName(u'topicsButtonLayout')
        self.topicsButtonsLayout.addStretch()
        self.topicsAddButton = QtGui.QPushButton(self.topicsPage)
        self.topicsAddButton.setIcon(build_icon(u':/songs/topic_add.png'))
        self.topicsAddButton.setObjectName(u'topicsAddButton')
        self.topicsButtonsLayout.addWidget(self.topicsAddButton)
        self.topicsEditButton = QtGui.QPushButton(self.topicsPage)
        self.topicsEditButton.setIcon(build_icon(u':/songs/topic_edit.png'))
        self.topicsEditButton.setObjectName(u'topicsEditButton')
        self.topicsButtonsLayout.addWidget(self.topicsEditButton)
        self.topicsDeleteButton = QtGui.QPushButton(self.topicsPage)
        self.topicsDeleteButton.setIcon(build_icon(u':/songs/topic_delete.png'))
        self.topicsDeleteButton.setObjectName(u'topicsDeleteButton')
        self.topicsButtonsLayout.addWidget(self.topicsDeleteButton)
        self.topicsLayout.addLayout(self.topicsButtonsLayout)
        self.stackedLayout.addWidget(self.topicsPage)
        # song books page
        self.booksPage = QtGui.QWidget(songMaintenanceDialog)
        self.booksPage.setObjectName(u'booksPage')
        self.booksLayout = QtGui.QVBoxLayout(self.booksPage)
        self.booksLayout.setObjectName(u'booksLayout')
        self.booksListWidget = QtGui.QListWidget(self.booksPage)
        self.booksListWidget.setObjectName(u'booksListWidget')
        self.booksLayout.addWidget(self.booksListWidget)
        self.booksButtonsLayout = QtGui.QHBoxLayout()
        self.booksButtonsLayout.setObjectName(u'booksButtonLayout')
        self.booksButtonsLayout.addStretch()
        self.booksAddButton = QtGui.QPushButton(self.booksPage)
        self.booksAddButton.setIcon(build_icon(u':/songs/book_add.png'))
        self.booksAddButton.setObjectName(u'booksAddButton')
        self.booksButtonsLayout.addWidget(self.booksAddButton)
        self.booksEditButton = QtGui.QPushButton(self.booksPage)
        self.booksEditButton.setIcon(build_icon(u':/songs/book_edit.png'))
        self.booksEditButton.setObjectName(u'booksEditButton')
        self.booksButtonsLayout.addWidget(self.booksEditButton)
        self.booksDeleteButton = QtGui.QPushButton(self.booksPage)
        self.booksDeleteButton.setIcon(build_icon(u':/songs/book_delete.png'))
        self.booksDeleteButton.setObjectName(u'booksDeleteButton')
        self.booksButtonsLayout.addWidget(self.booksDeleteButton)
        self.booksLayout.addLayout(self.booksButtonsLayout)
        self.stackedLayout.addWidget(self.booksPage)
        #
        self.dialogLayout.addLayout(self.stackedLayout, 0, 1)
        self.buttonBox = create_button_box(songMaintenanceDialog, u'buttonBox',
            [u'close'])
        self.dialogLayout.addWidget(self.buttonBox, 1, 0, 1, 2)
        self.retranslateUi(songMaintenanceDialog)
        self.stackedLayout.setCurrentIndex(0)
        QtCore.QObject.connect(self.typeListWidget,
            QtCore.SIGNAL(u'currentRowChanged(int)'),
            self.stackedLayout.setCurrentIndex)

    def retranslateUi(self, songMaintenanceDialog):
        songMaintenanceDialog.setWindowTitle(SongStrings.SongMaintenance)
        self.listItemAuthors.setText(SongStrings.Authors)
        self.listItemTopics.setText(SongStrings.Topics)
        self.listItemBooks.setText(SongStrings.SongBooks)
        self.authorsAddButton.setText(UiStrings().Add)
        self.authorsEditButton.setText(UiStrings().Edit)
        self.authorsDeleteButton.setText(UiStrings().Delete)
        self.topicsAddButton.setText(UiStrings().Add)
        self.topicsEditButton.setText(UiStrings().Edit)
        self.topicsDeleteButton.setText(UiStrings().Delete)
        self.booksAddButton.setText(UiStrings().Add)
        self.booksEditButton.setText(UiStrings().Edit)
        self.booksDeleteButton.setText(UiStrings().Delete)
        typeListWidth = max(self.fontMetrics().width(SongStrings.Authors),
            self.fontMetrics().width(SongStrings.Topics),
            self.fontMetrics().width(SongStrings.SongBooks))
        self.typeListWidget.setFixedWidth(typeListWidth +
            self.typeListWidget.iconSize().width() + 32)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The :mod:`languagemanager` module provides all the translation settings and
language file loading for OpenLP.
"""
import logging
import sys

from PyQt4 import QtCore, QtGui

from openlp.core.utils import AppLocation
from openlp.core.lib import translate
from openlp.core.lib.settings import Settings

log = logging.getLogger(__name__)

class LanguageManager(object):
    """
    Helper for Language selection
    """
    __qm_list__ = {}
    auto_language = False

    @staticmethod
    def get_translator(language):
        """
        Set up a translator to use in this instance of OpenLP

        ``language``
            The language to load into the translator
        """
        if LanguageManager.auto_language:
            language = QtCore.QLocale.system().name()
        lang_path = AppLocation.get_directory(AppLocation.LanguageDir)
        app_translator = QtCore.QTranslator()
        app_translator.load(language, lang_path)
        # A translator for buttons and other default strings provided by Qt.
        if sys.platform != u'win32' and sys.platform != u'darwin':
            lang_path = QtCore.QLibraryInfo.location(
                QtCore.QLibraryInfo.TranslationsPath)
        default_translator = QtCore.QTranslator()
        default_translator.load(u'qt_%s' % language, lang_path)
        return app_translator, default_translator

    @staticmethod
    def find_qm_files():
        """
        Find all available language files in this OpenLP install
        """
        log.debug(u'Translation files: %s', AppLocation.get_directory(
            AppLocation.LanguageDir))
        trans_dir = QtCore.QDir(AppLocation.get_directory(
            AppLocation.LanguageDir))
        file_names = trans_dir.entryList(QtCore.QStringList(u'*.qm'),
                QtCore.QDir.Files, QtCore.QDir.Name)
        # Remove qm files from the list which start with "qt_".
        file_names = file_names.filter(QtCore.QRegExp("^(?!qt_)"))
        for name in file_names:
            file_names.replaceInStrings(name, trans_dir.filePath(name))
        return file_names

    @staticmethod
    def language_name(qm_file):
        """
        Load the language name from a language file

        ``qm_file``
            The file to obtain the name from
        """
        translator = QtCore.QTranslator()
        translator.load(qm_file)
        return translator.translate('OpenLP.MainWindow', 'English',
            'Please add the name of your language here')

    @staticmethod
    def get_language():
        """
        Retrieve a saved language to use from settings
        """
        settings = Settings()
        language = unicode(settings.value(
            u'general/language', QtCore.QVariant(u'[en]')).toString())
        log.info(u'Language file: \'%s\' Loaded from conf file' % language)
        reg_ex = QtCore.QRegExp("^\[(.*)\]")
        if reg_ex.exactMatch(language):
            LanguageManager.auto_language = True
            language = reg_ex.cap(1)
        return language

    @staticmethod
    def set_language(action, message=True):
        """
        Set the language to translate OpenLP into

        ``action``
            The language menu option

        ``message``
            Display the message option
        """
        language = u'en'
        if action:
            action_name = unicode(action.objectName())
            if action_name == u'autoLanguageItem':
                LanguageManager.auto_language = True
            else:
                LanguageManager.auto_language = False
                qm_list = LanguageManager.get_qm_list()
                language = unicode(qm_list[action_name])
        if LanguageManager.auto_language:
            language = u'[%s]' % language
        Settings().setValue(
            u'general/language', QtCore.QVariant(language))
        log.info(u'Language file: \'%s\' written to conf file' % language)
        if message:
            QtGui.QMessageBox.information(None,
                translate('OpenLP.LanguageManager', 'Language'),
                translate('OpenLP.LanguageManager',
                    'Please restart OpenLP to use your new language setting.'))

    @staticmethod
    def init_qm_list():
        """
        Initialise the list of available translations
        """
        LanguageManager.__qm_list__ = {}
        qm_files = LanguageManager.find_qm_files()
        for counter, qmf in enumerate(qm_files):
            reg_ex = QtCore.QRegExp("^.*i18n/(.*).qm")
            if reg_ex.exactMatch(qmf):
                name = u'%s' % reg_ex.cap(1)
                LanguageManager.__qm_list__[u'%#2i %s' % (counter + 1,
                    LanguageManager.language_name(qmf))] = name

    @staticmethod
    def get_qm_list():
        """
        Return the list of available translations
        """
        if not LanguageManager.__qm_list__:
            LanguageManager.init_qm_list()
        return LanguageManager.__qm_list__



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging
import re
import shutil
import os

from PyQt4 import QtCore

from openlp.core.lib import Receiver, translate, check_directory_exists
from openlp.core.ui.wizard import WizardStrings
from openlp.core.utils import AppLocation
from openlp.plugins.songs.lib import clean_song, VerseType
from openlp.plugins.songs.lib.db import Song, Author, Topic, Book, MediaFile
from openlp.plugins.songs.lib.ui import SongStrings
from openlp.plugins.songs.lib.xml import SongXML

log = logging.getLogger(__name__)

class SongImport(QtCore.QObject):
    """
    Helper class for import a song from a third party source into OpenLP

    This class just takes the raw strings, and will work out for itself
    whether the authors etc already exist and add them or refer to them
    as necessary
    """
    @staticmethod
    def isValidSource(import_source):
        """
        Override this method to validate the source prior to import.
        """
        return True

    def __init__(self, manager, **kwargs):
        """
        Initialise and create defaults for properties

        ``manager``
            An instance of a SongManager, through which all database access is
            performed.

        """
        self.manager = manager
        QtCore.QObject.__init__(self)
        if u'filename' in kwargs:
            self.importSource = kwargs[u'filename']
        elif u'filenames' in kwargs:
            self.importSource = kwargs[u'filenames']
        elif u'folder' in kwargs:
            self.importSource = kwargs[u'folder']
        else:
            raise KeyError(
                u'Keyword arguments "filename[s]" or "folder" not supplied.')
        log.debug(self.importSource)
        self.importWizard = None
        self.song = None
        self.stopImportFlag = False
        self.setDefaults()
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'openlp_stop_wizard'), self.stopImport)

    def setDefaults(self):
        """
        Create defaults for properties - call this before each song
        if importing many songs at once to ensure a clean beginning
        """
        self.title = u''
        self.songNumber = u''
        self.alternateTitle = u''
        self.copyright = u''
        self.comments = u''
        self.themeName = u''
        self.ccliNumber = u''
        self.authors = []
        self.topics = []
        self.mediaFiles = []
        self.songBookName = u''
        self.songBookPub = u''
        self.verseOrderListGeneratedUseful = False
        self.verseOrderListGenerated = []
        self.verseOrderList = []
        self.verses = []
        self.verseCounts = {}
        self.copyrightString = unicode(translate(
            'SongsPlugin.SongImport', 'copyright'))

    def logError(self, filepath, reason=SongStrings.SongIncomplete):
        """
        This should be called, when a song could not be imported.

        ``filepath``
            This should be the file path if ``self.importSource`` is a list
            with different files. If it is not a list, but a single file (for
            instance a database), then this should be the song's title.

        ``reason``
            The reason why the import failed. The string should be as
            informative as possible.
        """
        self.setDefaults()
        if self.importWizard is None:
            return
        if self.importWizard.errorReportTextEdit.isHidden():
            self.importWizard.errorReportTextEdit.setText(
                translate('SongsPlugin.SongImport',
                'The following songs could not be imported:'))
            self.importWizard.errorReportTextEdit.setVisible(True)
            self.importWizard.errorCopyToButton.setVisible(True)
            self.importWizard.errorSaveToButton.setVisible(True)
        self.importWizard.errorReportTextEdit.append(
            u'- %s (%s)' % (filepath, reason))

    def stopImport(self):
        """
        Sets the flag for importers to stop their import
        """
        log.debug(u'Stopping songs import')
        self.stopImportFlag = True

    def register(self, import_wizard):
        self.importWizard = import_wizard

    def tidyText(self, text):
        """
        Get rid of some dodgy unicode and formatting characters we're not
        interested in. Some can be converted to ascii.
        """
        text = text.replace(u'\u2018', u'\'')
        text = text.replace(u'\u2019', u'\'')
        text = text.replace(u'\u201c', u'"')
        text = text.replace(u'\u201d', u'"')
        text = text.replace(u'\u2026', u'...')
        text = text.replace(u'\u2013', u'-')
        text = text.replace(u'\u2014', u'-')
        # Remove surplus blank lines, spaces, trailing/leading spaces
        text = re.sub(r'[ \t\v]+', u' ', text)
        text = re.sub(r' ?(\r\n?|\n) ?', u'\n', text)
        text = re.sub(r' ?(\n{5}|\f)+ ?', u'\f', text)
        return text

    def processSongText(self, text):
        verse_texts = text.split(u'\n\n')
        for verse_text in verse_texts:
            if verse_text.strip() != u'':
                self.processVerseText(verse_text.strip())

    def processVerseText(self, text):
        lines = text.split(u'\n')
        if text.lower().find(self.copyrightString) >= 0 \
            or text.find(unicode(SongStrings.CopyrightSymbol)) >= 0:
            copyright_found = False
            for line in lines:
                if (copyright_found or
                    line.lower().find(self.copyrightString) >= 0 or
                    line.find(unicode(SongStrings.CopyrightSymbol)) >= 0):
                    copyright_found = True
                    self.addCopyright(line)
                else:
                    self.parseAuthor(line)
            return
        if len(lines) == 1:
            self.parseAuthor(lines[0])
            return
        if not self.title:
            self.title = lines[0]
        self.addVerse(text)

    def addCopyright(self, copyright):
        """
        Build the copyright field
        """
        if self.copyright.find(copyright) >= 0:
            return
        if self.copyright != u'':
            self.copyright += ' '
        self.copyright += copyright

    def parseAuthor(self, text):
        """
        Add the author. OpenLP stores them individually so split by 'and', '&'
        and comma. However need to check for 'Mr and Mrs Smith' and turn it to
        'Mr Smith' and 'Mrs Smith'.
        """
        for author in text.split(u','):
            authors = author.split(u'&')
            for i in range(len(authors)):
                author2 = authors[i].strip()
                if author2.find(u' ') == -1 and i < len(authors) - 1:
                    author2 = author2 + u' ' \
                        + authors[i + 1].strip().split(u' ')[-1]
                if author2.endswith(u'.'):
                    author2 = author2[:-1]
                if author2:
                    self.addAuthor(author2)

    def addAuthor(self, author):
        """
        Add an author to the list
        """
        if author in self.authors:
            return
        self.authors.append(author)

    def addMediaFile(self, filename, weight=0):
        """
        Add a media file to the list
        """
        if filename in map(lambda x: x[0], self.mediaFiles):
            return
        self.mediaFiles.append((filename, weight))

    def addVerse(self, verse_text, verse_def=u'v', lang=None):
        """
        Add a verse. This is the whole verse, lines split by \\n. It will also
        attempt to detect duplicates. In this case it will just add to the verse
        order.

        ``verse_text``
            The text of the verse.

        ``verse_def``
            The verse tag can be v1/c1/b etc, or 'v' and 'c' (will count the
            verses/choruses itself) or None, where it will assume verse.

        ``lang``
            The language code (ISO-639) of the verse, for example *en* or *de*.

        """
        for (old_verse_def, old_verse, old_lang) in self.verses:
            if old_verse.strip() == verse_text.strip():
                self.verseOrderListGenerated.append(old_verse_def)
                self.verseOrderListGeneratedUseful = True
                return
        if verse_def[0] in self.verseCounts:
            self.verseCounts[verse_def[0]] += 1
        else:
            self.verseCounts[verse_def[0]] = 1
        if len(verse_def) == 1:
            verse_def += unicode(self.verseCounts[verse_def[0]])
        elif int(verse_def[1:]) > self.verseCounts[verse_def[0]]:
            self.verseCounts[verse_def[0]] = int(verse_def[1:])
        self.verses.append([verse_def, verse_text.rstrip(), lang])
        if verse_def not in self.verseOrderListGenerated:
            self.verseOrderListGenerated.append(verse_def)

    def repeatVerse(self):
        """
        Repeat the previous verse in the verse order
        """
        if self.verseOrderListGenerated:
            self.verseOrderListGenerated.append(
                self.verseOrderListGenerated[-1])
            self.verseOrderListGeneratedUseful = True

    def checkComplete(self):
        """
        Check the mandatory fields are entered (i.e. title and a verse)
        Author not checked here, if no author then "Author unknown" is
        automatically added
        """
        if not self.title or not self.verses:
            return False
        else:
            return True

    def finish(self):
        """
        All fields have been set to this song. Write the song to disk.
        """
        if not self.checkComplete():
            self.setDefaults()
            return False
        log.info(u'committing song %s to database', self.title)
        song = Song()
        song.title = self.title
        if self.importWizard is not None:
            self.importWizard.incrementProgressBar(
                WizardStrings.ImportingType % song.title)
        song.alternate_title = self.alternateTitle
        # Values will be set when cleaning the song.
        song.search_title = u''
        song.search_lyrics = u''
        song.verse_order = u''
        song.song_number = self.songNumber
        verses_changed_to_other = {}
        sxml = SongXML()
        other_count = 1
        for (verse_def, verse_text, lang) in self.verses:
            if verse_def[0].lower() in VerseType.Tags:
                verse_tag = verse_def[0].lower()
            else:
                new_verse_def = u'%s%d' % (VerseType.Tags[VerseType.Other],
                    other_count)
                verses_changed_to_other[verse_def] = new_verse_def
                other_count += 1
                verse_tag = VerseType.Tags[VerseType.Other]
                log.info(u'Versetype %s changing to %s', verse_def,
                    new_verse_def)
                verse_def = new_verse_def
            sxml.add_verse_to_lyrics(verse_tag, verse_def[1:], verse_text, lang)
        song.lyrics = unicode(sxml.extract_xml(), u'utf-8')
        if not self.verseOrderList and self.verseOrderListGeneratedUseful:
            self.verseOrderList = self.verseOrderListGenerated
        self.verseOrderList = map(lambda v: verses_changed_to_other.get(v, v),
            self.verseOrderList)
        song.verse_order = u' '.join(self.verseOrderList)
        song.copyright = self.copyright
        song.comments = self.comments
        song.theme_name = self.themeName
        song.ccli_number = self.ccliNumber
        for authortext in self.authors:
            author = self.manager.get_object_filtered(Author,
                Author.display_name == authortext)
            if not author:
                author = Author.populate(display_name=authortext,
                    last_name=authortext.split(u' ')[-1],
                    first_name=u' '.join(authortext.split(u' ')[:-1]))
            song.authors.append(author)
        if self.songBookName:
            song_book = self.manager.get_object_filtered(Book,
                Book.name == self.songBookName)
            if song_book is None:
                song_book = Book.populate(name=self.songBookName,
                    publisher=self.songBookPub)
            song.book = song_book
        for topictext in self.topics:
            if not topictext:
                continue
            topic = self.manager.get_object_filtered(Topic,
                Topic.name == topictext)
            if topic is None:
                topic = Topic.populate(name=topictext)
            song.topics.append(topic)
        # We need to save the song now, before adding the media files, so that
        # we know where to save the media files to.
        clean_song(self.manager, song)
        self.manager.save_object(song)
        # Now loop through the media files, copy them to the correct location,
        # and save the song again.
        for filename, weight in self.mediaFiles:
            media_file = self.manager.get_object_filtered(MediaFile,
                MediaFile.file_name == filename)
            if not media_file:
                if os.path.dirname(filename):
                    filename = self.copyMediaFile(song.id, filename)
                song.media_files.append(
                    MediaFile.populate(file_name=filename, weight=weight)
                )
        self.manager.save_object(song)
        self.setDefaults()
        return True

    def copyMediaFile(self, song_id, filename):
        """
        This method copies the media file to the correct location and returns
        the new file location.

        ``filename``
            The file to copy.
        """
        if not hasattr(self, u'save_path'):
            self.save_path = os.path.join(
                AppLocation.get_section_data_path(
                    self.importWizard.plugin.name),
                'audio', str(song_id))
        check_directory_exists(self.save_path)
        if not filename.startswith(self.save_path):
            oldfile, filename = filename, os.path.join(self.save_path,
                os.path.split(filename)[1])
            shutil.copyfile(oldfile, filename)
        return filename
